INSTALL.eh -- Installation instructions for the Error Handler
*************************************************************

Overview
========

The Error Handler uses the standard GNU installation model...
after unpacking the package there are just three easy steps to a 
complete installation:

   1) ./configure
   2) make
   3) make install

See the the file INSTALL for generic installation instructions.

Step Zero: unpack the distribution
==================================

The Error Handler package should come in a file named

   eh-n.n.tar.gz

where n.n is the version number.  This is a gzipped tar file.  To 
unpack the package, you will need the gunzip utility (or the GNU 
version of the zcat utility), and the tar utility.  You can use 
either one of the following command lines:

   gunzip eh-n.n.tar.gz; tar xvf eh-n.n.tar
   
or

   zcat eh-n.n.tar.gz | tar xvf -

or

   tar zxvf eh-n.n.tar.gz

The second form is slightly more efficient as the package file
remains compressed.  The first form first decompresses the
pacakge file on disk, then untars it.  The third form is available
with GNU tar.

The result should be the creation and population of the following
directory tree:

   eh-n.n
   eh-n.n/src
   eh-n.n/examples

Change to the eh-n.n directory before performing the rest of the 
installation steps.


Step One: ./configure
=====================

configure is a script that will perform some tests on your current
environment in an attempt to determine how to build the Error 
Handler.  The results of these tests are embedded in the various
Makefiles that are used by the package as well as the file 
src/ehconfig.h.  

The configure script has many options.  These can be listed by typing:

   ./configure --help

One option that I use frequently is the --prefix option.  This defaults
to /usr/local and represents the directory where the package (after being
built) will be installed.  Depending on your environment, you may need
to explicitly specify a prefix as follows:

   ./configure --prefix=/my/installation/directory

The Error Handler will create (if necessary) and use the following 
subdirectories under the $prefix directory:

   bin
   include
   lib

The configure script examines certain environmental variables to help
it determine what compilers and librarians to use.  It would be 
extremely useful to have the following environmental variables 
properly defined.

   CXX      -- should invoke your C++ compiler with exception 
                     handling enabled
   CXXFLAGS -- the flags that should be passed to $CXX

   CC       -- should invoke your C compiler
   CFLAGS   -- the flags that should be passed to $CC

   AR       -- should invoke your object module librarian
   ARFLAGS  -- the flags that should be passed to $AR

The Error Handler prefers to use a C++ compiler.  Therefore, if it 
detects a C++ compiler (possibly using the $CXX environmental 
variable), this will supercede a C compiler that may be present.

configure works with an optional 'site configuration file' 
(config.site) which provides a specific environment which configure 
then uses while it is executing.  This file can set values for CXX, 
AR, etc., to known successful values for specific sites.  Several 
sample site configuration files (config.site.*) are provided with 
the Error Handler.  You may need to customize the site configuration
file to reflect your own setup.  See the comments in the 
config.site.* files for more details.

The ./configure script needs to be told to use a particular site 
configuration file.  There are a couple of ways to do this.  One way 
One way is to set the environmental variable CONFIG_SITE to its 
pathname,
then run ./configure:

   CONFIG_SITE=/.../eh-n.n/config.site.xxxx
   export CONFIG_SITE
   ./configure.

Another way is to copy the desired site configuration file to 
/usr/local/share/config.site, and then run ./configure:

   cp config.site.xxxx /usr/local/share/config.site
   ./configure

Finally, your installation prefix may be something other than 
/usr/local.  In this case, copy the desired site configuration 
file to $prefix/share/config.site, and then run ./configure with 
the --prefix option:

   cp config.site.xxxx /my/installation/directory/share/config.site
   ./configure --prefix=/my/installation/directory

Don't forget to customize your site configuration file to reflect your
own environment.  Also, when running configure, you should *never* get

   checking whether cross-compiling... yes

(unless you're really cross compiling :).  If you get a 'yes' for cross-
compiling, this is an idicator that something is mis-configured.


Step Two: make
==============

Typing make will cause the library to be compiled and built.  


Step Three: make install
========================

Typing make install will cause the Error Handler library to be installed
in the $prefix directory.  By default this is /usr/local.

It also builds the examples found in the eh-n.n/examples directory.


What's Next?
============

Go through the examples.  The examples constitute a tutorial on the 
Error Handler and can also be used to ensure that the Error Handler 
has been built and installed properly.  The examples demonstrate how
to use the various Error Handler features.  There is an initial emphasis
on resource tracking as this feature can be applied to existing code 
with only minor modifications.

The examples also illustrate function tracing, throwing exceptions,
the management of error messages and error log "channels."   They 
finally demonstrate how to activate Control-C handling and how to
simulate a Control-C in your code to assist in testing.

README.eh contains an overview of the Error Handler's features.


Happy Error Handling!

Christopher Cobb
Cobb Software Engineering Guild, Inc.
ccobb@cseg.com
