/***

ansi_mem.c -- illustrate ansi memory usage tracking

    Copyright (C) 1997  CSEG, Inc., All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   
 
    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.
 
***/

#include <ehmem.h>           /* include this instead of <eh.h> */

static void malloc_and_free();
static void malloc_and_no_free();
static void calloc_and_free();
static void calloc_and_no_free();
static void malloc_realloc_and_free();
static void malloc_realloc_and_no_free();
static void calloc_realloc_and_free();
static void calloc_realloc_and_no_free();
static void realloc_null_and_free();
static void realloc_null_and_no_free();
static void new_and_delete();
static void new_and_no_delete();

int
main(int argc, char**argv)
{
   EHMAIN(argc,argv) { /* initialize Error Handler */

      int i;
      for (i = 3; i--; ) /* to illustrate pass counting */
      {
         malloc_and_free();
         malloc_and_no_free();

         calloc_and_free();
         calloc_and_no_free();

         malloc_realloc_and_free();
         malloc_realloc_and_no_free();

         calloc_realloc_and_free();
         calloc_realloc_and_no_free();

         realloc_null_and_free();
         realloc_null_and_no_free();

         new_and_delete();
         new_and_no_delete();
      }

   } EHENDMAIN;

   EHDONE(0);
}

static
void
malloc_and_free()
{
   char *pc = (char*)malloc(37);
   if (pc) free(pc);
}


static
void
malloc_and_no_free()
{
   char *pc = (char*)malloc(37);
}


static
void
calloc_and_free()
{
   char *pc = (char*)calloc(3,7);
   if (pc) free(pc);
}


static
void
calloc_and_no_free()
{
   char *pc = (char*)calloc(3,7);
}


static
void
malloc_realloc_and_free()
{
   char *pc = (char*)malloc(37);
   pc = (char*)realloc(pc,3377);
   if (pc) free(pc);
}


static
void
malloc_realloc_and_no_free()
{
   char *pc = (char*)malloc(37);
   pc = (char*)realloc(pc,3377);
}


static
void
calloc_realloc_and_free()
{
   char *pc = (char*)calloc(33,77);
   pc = (char*)realloc(pc,3377);
   if (pc) free(pc);
}


static
void
calloc_realloc_and_no_free()
{
   char *pc = (char*)calloc(33,77);
   pc = (char*)realloc(pc,3377);
}


static
void
realloc_null_and_free()
{
   char *pc = (char*)realloc(NULL,3377);
   if (pc) free(pc);
}


static
void
realloc_null_and_no_free()
{
   char *pc = (char*)realloc(NULL,3377);
}


static
void
new_and_delete()
{
#ifdef __cplusplus
   char *pc = new char[37];
   if (pc) delete pc;
#endif
}


static
void
new_and_no_delete()
{
#ifdef __cplusplus
   char *pc = new char[37];
#endif
}


