/***

ansi_res.c -- illustrate ansi memory and file resource tracking

    Copyright (C) 1997  CSEG, Inc., All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   
 
    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.
 
***/

#include <ehmemtrk.h>           /* for function-tracing memory tracking */
#include <ehfiltrk.h>           /* for function-tracing file tracking */

static void memory_tracking();
static void file_tracking();

static void malloc_and_no_free();
static void calloc_and_no_free();
static void malloc_realloc_and_no_free();
static void calloc_realloc_and_no_free();
static void realloc_null_and_no_free();
static void new_and_delete();
static void new_and_no_delete();

static void fopen_and_no_fclose();
static void tmpfile_and_no_fclose();
static void fopen_freopen_and_no_fclose();
static void freopen_stdin_and_no_fclose();

int
main(int argc, char**argv)
{
   int i;

   EHMAIN(argc,argv) { /* initialize Error Handler */

      for (i = 2; i--; ) /* to illustrate pass counting */
      {
         memory_tracking();
         file_tracking();
      }

   } EHENDMAIN;

   EHDONE(0);
}

static
void
memory_tracking()
{
   EHTRACE(memory_tracking);

   malloc_and_no_free();

   //calloc_and_no_free();

   //malloc_realloc_and_no_free();

   //calloc_realloc_and_no_free();

   //realloc_null_and_no_free();

   //new_and_delete();
   new_and_no_delete();
}

static
void
file_tracking()
{
   EHTRACE(file_tracking);

   fopen_and_no_fclose();

   //tmpfile_and_no_fclose();

   fopen_freopen_and_no_fclose();

   //freopen_stdin_and_no_fclose();
}

static
void
malloc_and_no_free()
{
   char *pc;
   EHTRACE(malloc_and_no_free);
   pc = (char*)malloc(37);
}                     // no EHLEAVE()


static
void
calloc_and_no_free()
{
   char *pc;
   EHTRACE(calloc_and_no_free);
   pc = (char*)calloc(3,7);
}


static
void
malloc_realloc_and_no_free()
{
   char *pc;
   EHTRACE(malloc_realloc_and_no_free);
   pc = (char*)malloc(37);
   pc = (char*)realloc(pc,3377);
}


static
void
calloc_realloc_and_no_free()
{
   char *pc;
   EHTRACE(calloc_realloc_and_no_free);
   pc = (char*)calloc(33,77);
   pc = (char*)realloc(pc,3377);
}


static
void
realloc_null_and_no_free()
{
   char *pc;
   EHTRACE(realloc_null_and_no_free);
   pc = (char*)realloc(NULL,3377);
}


static
void
new_and_delete()
{
   char *pc;
   EHTRACE(new_and_delete);
#ifdef __cplusplus
   pc = new char[37];
   if (pc) delete pc;
#endif
   EHLEAVE();
}


static
void
new_and_no_delete()
{
   char *pc;
   EHTRACE(new_and_no_delete);
#ifdef __cplusplus    // don't do this in C
   pc = new char[37];
#endif
   EHLEAVE();         // tell the Error Handler we are leaving
}


static
void
fopen_and_no_fclose()
{
   FILE *pFile;
   EHTRACE(fopen_and_no_fclose);
   pFile = fopen("ansi_res.2", "w");
}


static
void
tmpfile_and_no_fclose()
{
   FILE *pFile;
   EHTRACE(tmpfile_and_no_fclose);
   pFile = tmpfile();
}


static
void
fopen_freopen_and_no_fclose()
{
   FILE *pFile;
   EHTRACE(fopen_freopen_and_no_fclose);
   pFile = fopen("ansi_res.5", "w");
   pFile = freopen("ansi_res.6", "w", pFile);
}


static
void
freopen_stdin_and_no_fclose()
{
   FILE *pFile;
   EHTRACE(freopen_stdin_and_no_fclose);
   pFile = freopen("ansi_res.8", "w", stdin);
}



