/***

showme.c -- a sample program which displays itself

    Copyright (C) 1997  CSEG, Inc., All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   
 
    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.
 
***/

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <ehmemtrk.h>        /* memory usage tracking */
#include <ehfiltrk.h>        /* file usage tracking */

/***
This function reads a file into memory,
then prints it to stdout 
***/

static
void
showme(char *pcFileName)
{
   /*** this is the initialization section ***/

   FILE *volatile pFile = NULL; /* initialize resource handles */
   char *volatile pcBuf = NULL;

   /*** this is main section ***/

   EHTRY(showme) {           

      struct stat Stat;           /* this section has its own scope */
      int iStatResult;            /* local variables go here */

      /* get the file size */
      iStatResult = stat(pcFileName, &Stat);
      if (iStatResult == -1)           /* if an error occurs */
      {
         puts("showme: could not stat the file");
         break;                        /* no need to continue */
      }

      /* allocate a buffer */
      pcBuf = new char[Stat.st_size + 1];
      if (pcBuf == NULL)               /* if an error occurs */
      {
         puts("showme: could not allocate memory");
         break;                        /* no need to continue */
      }

      /* before opening let's make sure the file is writable */
      if (!(Stat.st_mode & S_IWUSR))   /* file not writable by user */
      {
         puts("showme: file not writable");
         break;                        /* no need to continue */
      }

      /* open the file */
      pFile = fopen(pcFileName, "r+"); /* open for update */
      if (pFile == NULL)               /* if an error occurs */
      {
         puts("showme: could not open the file");
         break;                        /* no need to continue */
      }

      /* read it into the buffer */
      if (fread(pcBuf, Stat.st_size, 1, pFile) < 0)
      {
         puts("showme: could not read the file");
         break;
      }

      pcBuf[Stat.st_size] = 0;          /* null-terminate the buffer */

      puts(pcBuf);                      /* print out the file */

   } EHCATCH;

   /*** this is the clean up section ***/

   if (pFile)                  /* if resource was obtained */
      fclose(pFile);           /* clean up */

   if (pcBuf)                  /* if resource was obtained */
      delete [] pcBuf;         /* clean up */

   EHLEAVE();                  /* track line number of function exit */
}

main(int argc, char **argv)
{
   EHMAIN(argc, argv) {

      showme("showme.c");

   } EHENDMAIN;

   EHDONE(0);
}
