/***

t_fildes.c -- test driver for file usage tracking routines

    Copyright (C) 1997  CSEG, Inc., All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   
 
    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.
 
***/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <eh.h>
#include "lrfildes.h"

static void creat_and_close();
static void creat_and_no_close();
static void open_and_close();
static void open_and_no_close();
static void open_dup_and_close();
static void open_dup_and_no_close();

int
main(int argc, char**argv)
{
   EHMAIN(argc, argv)
   {

      creat_and_close();
      creat_and_no_close();

      open_and_close();
      open_and_no_close();

      open_dup_and_close();
      open_dup_and_no_close();

   } EHENDMAIN;

   EHRETURN(ehError());
}


static
void
creat_and_close()
{
   volatile int iFilDes = -1;

   EHTRY(creat_and_close) {

      iFilDes = creat("t_fildes.1", S_IRWXU);

   } EHCATCH;

   if (iFilDes != -1) close(iFilDes);
}


static
void
creat_and_no_close()
{
   volatile int iFilDes = -1;

   EHTRACE(creat_and_no_close);

   iFilDes = creat("t_fildes.2", S_IRWXU);
}

static
void
open_and_close()
{
   volatile int iFilDes = -1;

   EHTRY(open_and_close) {

      iFilDes = open("t_fildes.1", O_RDONLY);

   } EHCATCH;

   if (iFilDes != -1) close(iFilDes);
}


static
void
open_and_no_close()
{
   volatile int iFilDes = -1;

   EHTRACE(open_and_no_close);

   iFilDes = open("t_fildes.1", O_RDONLY);
}

static
void
open_dup_and_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;

   EHTRY(open_dup_and_close) {

      iFilDes1 = open("t_fildes.1", O_RDONLY);
      iFilDes2 = dup(iFilDes1);

   } EHCATCH;

   if (iFilDes2 != -1) close(iFilDes2);
   if (iFilDes1 != -1) close(iFilDes1);
}


static
void
open_dup_and_no_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;

   EHTRACE(open_dup_and_no_close);

   iFilDes1 = open("t_fildes.1", O_RDONLY);
   iFilDes2 = dup(iFilDes1);
}

