/***

lrdirent.c -- Local Resources, directory tracking

    Copyright (C) 1997  CSEG, Inc., All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   
 
    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.
 
***/

#define LR_NO_DIRENT_TRACKING 1
#include "lrdirent.h" /* pick up function prototypes */
#include "lrinit.h"   /* pick up resource types */
#include "lrerrs.h"   /* local error codes */

/***
_lrOpendir()
Track opendir() calls.
***/

DIR* /* null if directory could not be opened, otherwise a valid DIR* */
_lrOpendir(               /* track opendir() calls */
   const char *filename,  /* file from which this function was called */
   const char *funcname,  /* function from which this function was called */
   int line,              /* line on which this function was called */
   const char *dirname)   /* name of directory to open */
{
   DIR *volatile pDir = NULL;  /* initialize to a non-valid value */

   EHTRY(_lrOpendir) {       /* begin function body proper of catcher */

      pDir = opendir(dirname); /* attempt to obtain resource */
      if (pDir == NULL)        /* if unsuccessful, print error */
      {
         ehPError(filename, funcname, line, EH_IS_ERROR, LR_OPENDIR);
         break;                /* out to section three */
      }

      ehResObt(filename,funcname,line,&ehResTrk,
            (int)LR_RES_DIRENT, dirname,
            (pfv_ulv)closedir, 1L, 
            (ulong)pDir);

   } EHCATCH;

   /* recovery */
   if (ehError() && pDir)
   {
      closedir(pDir);
      EHRETURN((DIR*)NULL);
   }

   EHRETURN(pDir);
} /* _lrOpendir */


/***
_lrClosedir()
Track closedir() calls.
***/

void
_lrClosedir(const char *filename,
   const char *funcname,
   int line,
   DIR*pDir)
{
   EHTRY(_lrClosedir) {
   /***
      no body but we need clean up section
      so that we are protected from exceptions
      being thrown...exception throwing from
      within a clean-up section is inhibited.
   ***/
   } EHCATCH;

   /* mark the resource and release it */
   ehResRel(filename,funcname,line,&ehResTrk,
         (int)LR_RES_DIRENT, (pfv_ulv)closedir, 1L, 
         (ulong)pDir);

   EHLEAVE();
} /* _lrClosedir */


