/***

lrfiltrk.c -- Local Resources, file stream tracking

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char*rcsid =
       "$Id: lrfiltrk.c,v 1.7 1996/03/08 07:35:10 ccobb Exp $";

#define EH_NO_FILE_TRACKING 1
#define LR_NO_FILE_TRACKING 1
#include "lrfiltrk.h"

EHBEGINEXTERNC
#include <string.h>
EHENDEXTERNC

#include <eherrs.h>
#include "lrerrs.h"

EHFILENAME(lrfiltrk.c); /* needed by compilers that don't support __FILE__ */

/***
_lrFdopen()
Track file stream allocation with fdopen().
This function is an fdopen() wrapper.  
***/

FILE *
_lrFdopen(
   const char *pcFile,
   const char *pcFunc,
   int         iLine,
   int         iFilDes,
   const char *pcType)
{
   FILE *volatile pFile = NULL;

   EHTRY(_lrFdopen)
   {
      pFile = fdopen(iFilDes, pcType); /* obtain the file stream ptr */
      if (pFile == NULL)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_FDOPEN);
         break;
      }

      ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)EH_RES_FILESTR, "fdopen'd file stream",
            (pfv_ulv)fclose, 1L, 
            (ulong)pFile);

   } EHCATCH;

   /* recovery */
   if (ehError() && pFile)
   {
      fclose(pFile);
      EHRETURN((FILE*)NULL);
   }

   EHRETURN(pFile);
} /* _lrFdopen */


