/***

lrinit.c -- Local Resources, initialization

    Copyright (C) 1997  CSEG, Inc., All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   
 
    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.
 
***/

#include "lrinit.h"   /* "local resource" initialization header */

/***
termination function 
***/

static
void
lrTerm() /* local resource termination function */
{
   EHTRY(lrTerm) {} EHCATCH;      /* catcher function */

   /* close file descriptors 2, 1, and 0 if they are open */
   if (ehResVerify(&ehResTrk, (ulong)2, (int)LR_RES_FILDES))
   {
      ehResRel(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)LR_RES_FILDES, (pfv_ulv)0, 1L,
            (ulong)2);
   }
   if (ehResVerify(&ehResTrk, (ulong)1, (int)LR_RES_FILDES))
   {
      ehResRel(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)LR_RES_FILDES, (pfv_ulv)0, 1L,
            (ulong)1);
   }
   if (ehResVerify(&ehResTrk, (ulong)0, (int)LR_RES_FILDES))
   {
      ehResRel(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)LR_RES_FILDES, (pfv_ulv)0, 1L,
            (ulong)0);
   }

   EHLEAVE();            /* optional for C++, required for C */
} /* lrTerm */


/***
initialization function
***/

static
void
lrInit() /* local resource initialization function */
{
   volatile bool gotStdin  = false;
   volatile bool gotStdout = false;

   EHTRY(lrInit) {

      /* first set up the termination routine */
      ehAtExit(lrTerm);  /* called at EH termination time */

      /* now register file descriptors 0, 1, and 2 */
      gotStdin = ehResObt(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)LR_RES_FILDES, "fd 0 (stdin)",
            (pfv_ulv)0, EH_RES_ALLOW_ZERO | 1L, 
            (ulong)0);

      gotStdout = ehResObt(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)LR_RES_FILDES, "fd 1 (stdout)",
            (pfv_ulv)0, EH_RES_ALLOW_ZERO | 1L, 
            (ulong)1);

      ehResObt(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)LR_RES_FILDES, "fd 2 (stderr)",
            (pfv_ulv)0, EH_RES_ALLOW_ZERO | 1L, 
            (ulong)2);

   } EHCATCH;

   if (ehError())
   {
      if (gotStdout)
      {
         ehResRel(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
               (int)LR_RES_FILDES, (pfv_ulv)0, EH_RES_ALLOW_ZERO | 1L,
               (ulong)1);
      }
      if (gotStdin)
      {
         ehResRel(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
               (int)LR_RES_FILDES, (pfv_ulv)0, EH_RES_ALLOW_ZERO | 1L,
               (ulong)0);
      }
   }

   EHLEAVE();
} /* lrInit */


/***
external initialization vector
***/

/* pre-empt the Error Handler's external initialization vector */
void (*pfvEhExtInit)() = lrInit;


