/***

lrmemtrk.c -- Local Resources, memory tracking

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char*rcsid =
       "$Id: lrmemtrk.c,v 1.7 1996/03/08 07:35:10 ccobb Exp $";

#define EH_NO_MEMORY_TRACKING 1 /* to prevent ehmemtrk.h recursion */
#define LR_NO_MEMORY_TRACKING 1 /* to prevent lrmemtrk.h recursion */
#include "lrmemtrk.h"           /* this also pulls in <ehmemtrk.h> */
#include "lrerrs.h"             /* local error codes */
#include <eherrs.h>             /* Error Handler error codes */

EHBEGINEXTERNC
#include <string.h>
EHENDEXTERNC

/* needed by compilers that don't support __FILE__ */
EHFILENAME(lrmemtrk.c);

/***
_lrStrdup()
Track dynamic memory allocation with strdup().
This function is a strdup() wrapper.  
***/

char *
_lrStrdup(
   const char *pcFile,
   const char *pcFunc,
   int         iLine,
   const char *pcString)
{
   char *volatile pcMem = NULL;

   EHTRY(_lrStrdup)
   {
      /***
         In order to allow for bounds checking,
         (which the Error Handler automatically
         performs on all dynamically allocated memory),
         we need to re-implement the functionality of
         strdup() here.
      ***/

      size_t szBlk = strlen(pcString) + 1;

      /* get EH_SZBOUNDBLK extra bytes for bounds checking */
      pcMem = (char*)malloc(szBlk + EH_SZBOUNDBLK);
      if (pcMem == NULL)
      {
         ehPError(pcFile, pcFunc, iLine, EH_IS_FATAL, LR_STRDUP);
         break;
      }
      strcpy(pcMem, pcString); /* perform the strdup() */

      /* write the bounds checking pattern */
      ehBndSet(pcMem + szBlk, EH_SZBOUNDBLK);

      ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)EH_RES_C_MEM, "strdup'ed memory",
            (pfv_ulv)free, 2L, 
            (ulong)pcMem,
            (ulong)szBlk);

   } EHCATCH;

   if (ehError() && pcMem)
   {
      free(pcMem);
      EHRETURN((char*)NULL);
   }

   EHRETURN(pcMem);
} /* _lrStrdup */


