/***

t_fildes.c -- rigourous test driver for file usage tracking routines

    Copyright (C) 1997  CSEG, Inc., All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   
 
    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.
 
***/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "lrfildes.h"
#include "lrfiltrk.h"

static void creat_and_close();
static void creat_dup2_and_close();
static void creat_dup2_and_no_close();
static void creat_and_no_close();

static void open_and_close();
static void open_dup2_and_close();
static void open_dup2_and_no_close();
static void open_and_no_close();

static void dup_and_close();
static void dup_dup2_and_close();
static void dup_dup2_and_no_close();
static void dup_and_no_close();

static void fileno_and_close();
static void fileno_dup2_and_close();
static void fileno_dup2_and_no_close();
static void fileno_and_no_close();

static void pipe_and_close_close();
static void pipe_and_no_close_close();
static void pipe_and_close_no_close();
static void pipe_and_no_close_no_close();

int
main(int argc, char**argv)
{
   EHMAIN(argc, argv)
   {

      creat_and_close();
      creat_dup2_and_close();
      creat_dup2_and_no_close();
      creat_and_no_close();

      open_and_close();
      open_dup2_and_close();
      open_dup2_and_no_close();
      open_and_no_close();

      dup_and_close();
      dup_dup2_and_close();
      dup_dup2_and_no_close();
      dup_and_no_close();

      fileno_and_close();
      fileno_dup2_and_close();

      fileno_dup2_and_no_close();
      fileno_and_no_close();

      pipe_and_close_close();
      pipe_and_no_close_close();
      pipe_and_close_no_close();
      pipe_and_no_close_no_close();

   } EHENDMAIN;

   EHRETURN(ehError());
}


static
void
creat_and_close()
{
   volatile int iFilDes = -1;

   EHTRY(creat_and_close) {

      iFilDes = creat("t_fildes.1", S_IRWXU);

   } EHCATCH;

   if (iFilDes != -1) close(iFilDes);
}


static
void
creat_dup2_and_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHTRY(creat_dup2_and_close) {

      iFilDes1 = creat("t_fildes.2", S_IRWXU);
      iFilDes2 = creat("t_fildes.3", S_IRWXU);
      iFilDes3 = dup2(iFilDes1, iFilDes2);

   } EHCATCH;

   if (iFilDes3 != -1) close(iFilDes3);
   if (iFilDes1 != -1) close(iFilDes1);
}


static
void
creat_dup2_and_no_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHTRY(creat_dup2_and_no_close) {

      iFilDes1 = creat("t_fildes.4", S_IRWXU);
      iFilDes2 = creat("t_fildes.5", S_IRWXU);
      iFilDes3 = dup2(iFilDes1, iFilDes2);

   } EHCATCH;

   if (iFilDes3 != -1) close(iFilDes3);
}


static
void
creat_and_no_close()
{
   volatile int iFilDes = -1;

   EHTRACE(creat_and_no_close);

   iFilDes = creat("t_fildes.6", S_IRWXU);
}



static
void
open_and_close()
{
   volatile int iFilDes = -1;

   EHTRY(open_and_close) {

      iFilDes = open("t_fildes.1", O_RDWR);

   } EHCATCH;

   if (iFilDes != -1) close(iFilDes);
}


static
void
open_dup2_and_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHTRY(open_dup2_and_close)
   {

      iFilDes1 = open("t_fildes.2", O_RDWR);
      iFilDes2 = creat("t_fildes.9", S_IRWXU);
      iFilDes3 = dup2(iFilDes1, iFilDes2);

   } EHCATCH;

   if (iFilDes3 != -1) close(iFilDes3);
   if (iFilDes1 != -1) close(iFilDes1);
}


static
void
open_dup2_and_no_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHTRY(open_dup2_and_no_close)
   {

      iFilDes1 = open("t_fildes.3", O_RDWR);
      iFilDes2 = creat("t_fildes.D", S_IRWXU);
      iFilDes3 = dup2(iFilDes1, iFilDes2);

   } EHCATCH;

   if (iFilDes3 != -1) close(iFilDes3);
}


static
void
open_and_no_close()
{
   volatile int iFilDes = -1;

   EHTRACE(open_and_no_close);

   iFilDes = open("t_fildes.4", O_RDWR);
}


static
void
dup_and_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;

   EHTRY(dup_and_close) {

      iFilDes1 = creat("t_fildes.F", S_IRWXU); /* 0x8 obtained */
      iFilDes2 = dup(iFilDes1);

   } EHCATCH;

   if (iFilDes2 != -1) close(iFilDes2);
   if (iFilDes1 != -1) close(iFilDes1); /* 0x8 released */
}


static
void
dup_dup2_and_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHTRY(dup_dup2_and_close)
   {

      iFilDes1 = creat("t_fildes.G", S_IRWXU); /* 0x8 obtained */
      iFilDes2 = dup(iFilDes1);
      iFilDes3 = dup2(iFilDes2, iFilDes1); /* 0x8 reobtained */

   } EHCATCH;

   if (iFilDes3 != -1) close(iFilDes3); /* 0x8 released */
   if (iFilDes2 != -1) close(iFilDes2);
}


static
void
dup_dup2_and_no_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHTRY(dup_dup2_and_no_close)
   {

      iFilDes1 = creat("t_fildes.G", S_IRWXU); /* 0x8 obtained */
      iFilDes2 = dup(iFilDes1);
      iFilDes3 = dup2(iFilDes2, iFilDes1); /* 0x8 reobtained */

   } EHCATCH;

   if (iFilDes3 != -1) close(iFilDes3); /* 0x8 released */
}


static
void
dup_and_no_close()
{
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHTRY(dup_and_no_close)
   {

      iFilDes1 = creat("t_fildes.G", S_IRWXU); /* 0x8 obtained */
      iFilDes2 = dup(iFilDes1);

   } EHCATCH;

   if (iFilDes1 != -1) close(iFilDes1); /* 0x8 released */
}


static
void
fileno_and_close()
{
   FILE *volatile pFile = NULL;
   volatile int iFilDes = -1;

   EHTRY(fileno_and_close)
   {

      pFile = fopen("t_fildes.J", "w");
      iFilDes = fileno(pFile); /* 0x8 obtained */

   } EHCATCH;

   if (pFile)         fclose(pFile); /* closes both */
}


static
void
fileno_dup2_and_close()
{
   FILE *volatile pFile = NULL;
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHTRY(fileno_dup2_and_close)
   {

      pFile = fopen("t_fildes.K", "w");
      iFilDes1 = fileno(pFile);                     /* 0x8 obtained */
      iFilDes2 = creat("t_fildes.L", S_IRWXU);
      iFilDes3 = dup2(iFilDes2, iFilDes1);          /* 0x8 reobtained */

   } EHCATCH;

   if (iFilDes2 != -1) close(iFilDes2);
   if (pFile)          fclose(pFile);               /* closes both */
}


static
void
fileno_dup2_and_no_close()
{
   FILE *volatile pFile = NULL;
   volatile int iFilDes1 = -1;
   volatile int iFilDes2 = -1;
   volatile int iFilDes3 = -1;

   EHTRY(fileno_dup2_and_no_close)
   {

      pFile = fopen("t_fildes.M", "w");
      iFilDes1 = fileno(pFile);                      /* 0x8 obtained */
      iFilDes2 = creat("t_fildes.N", S_IRWXU);
      iFilDes3 = dup2(iFilDes1, iFilDes2);

   } EHCATCH;

   if (iFilDes3 != -1) close(iFilDes3);
}


static
void
fileno_and_no_close()
{
   FILE *volatile pFile = NULL;
   volatile int iFilDes = -1;

   EHTRACE(fileno_and_no_close);

   pFile = fopen("t_fildes.O", "w");
   iFilDes = fileno(pFile);

}

   //} EHCATCH;

static
void
pipe_and_close_close()
{
   volatile int iRetCode = -1;
   int aiFilDes[2];

   EHTRY(pipe_and_close_close) {

      iRetCode = pipe(aiFilDes);

   } EHCATCH;

   if (iRetCode == 0)
   {
      close(aiFilDes[0]);
      close(aiFilDes[1]);
   }
}


static
void
pipe_and_no_close_close()
{
   volatile int iRetCode = -1;
   int aiFilDes[2];

   EHTRY(pipe_and_no_close_close) {

      iRetCode = pipe(aiFilDes);

   } EHCATCH;

   if (iRetCode == 0)
   {
      close(aiFilDes[1]);
   }
}


static
void
pipe_and_close_no_close()
{
   volatile int iRetCode = -1;
   int aiFilDes[2];

   EHTRY(pipe_and_no_close_close) {

      iRetCode = pipe(aiFilDes);

   } EHCATCH;

   if (iRetCode == 0)
   {
      close(aiFilDes[0]);
   }
}


static
void
pipe_and_no_close_no_close()
{
   int aiFilDes[2];

   EHTRACE(pipe_and_no_close_no_close);

   pipe(aiFilDes);
}


