/***

eh.h -- Error Handler, public definitions

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

#ifndef EH_H
#define EH_H 1
static const char*eh_h =
       "$Id: eh.h,v 1.21 1996/05/16 20:54:04 ccobb Exp $";

#define EH_SUPPLY_FUNCTION_NAME 1 /* programmer supplies function names */
/***
If the complier does not provide the
current function name in the preprocessor
definition __FUNC__, then EH_SUPPLY_FUNCTION_NAME
must be defined as 1.  This forces the programmer 
to provide explicit function names with
the EHTRACE(function_name) and EHTRY(function_name)
macros.  This is also the most portable method, because
the __FUNC__ macro is not in the C standard definition
and is therefore not universally supported.

However, if EH_SUPPLY_FUNCTION_NAME is left undefined
AND the current compiler supports __FUNC__, then the
EHTRACE() and EHTRY() macros can be used without
explicitly providing the current function's name, as
it will be picked up using __FUNC__.  This is easier
to use, but is less portable, and is therefore not
recommended.

The #if statement immediately below may use 0 or 1 as follows:
0 (recommended) means that function names must be supplied by programmer.
1 (not recommended) may only be used when the compiler supports __FUNC__
  and means that function names may be ommitted in EHTRACE() and EHTRY().
***/
#if 0 && defined(__FUNC__)
#undef EH_SUPPLY_FUNCTION_NAME    /* function names picked up from __FUNC__ */
#endif

#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE 1
#endif

#ifndef   EHENV_H
#include "ehenv.h"
#endif

/***
 ***
 *** Required include files
 ***
 ***/

EHBEGINEXTERNC

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#ifndef ehEXCEPTIONHANDLING
#include <setjmp.h>
#endif

/*** environment-specific-dependencies ***/

/* variable-argument support */
#if ehSUNOS
#define SUPPORTS_STDARG 0
#include <varargs.h>
#else
#include <stdarg.h>
#define SUPPORTS_STDARG 1
#endif

#if !ehMX && !ehSTMC
#include <sys/types.h>
#endif

EHENDEXTERNC

/***
 ***
 *** Common definitions
 ***
 ***/

/*** dual-method function argument definitions ***/

/* non-variable argument list */
#if !ehSUNC
#define A(x) x
#define A0() (void)
#define A1(t1, n1) (t1 n1)
#define A2(t1, n1, t2, n2) (t1 n1, t2 n2)
#define A3(t1, n1, t2, n2, t3, n3) (t1 n1, t2 n2, t3 n3)
#define A4(t1,n1,t2,n2,t3,n3,t4,n4)\
      (t1 n1,t2 n2,t3 n3,t4 n4)
#define A5(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5)\
      (t1 n1,t2 n2,t3 n3,t4 n4,t5 n5)
#define A6(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6)\
      (t1 n1,t2 n2,t3 n3,t4 n4,t5 n5,t6 n6)
#define A7(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,t7,n7)\
      (t1 n1,t2 n2,t3 n3,t4 n4,t5 n5,t6 n6,t7 n7)
#define A8(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,t7,n7,t8,n8)\
      (t1 n1,t2 n2,t3 n3,t4 n4,t5 n5,t6 n6,t7 n7,t8 n8)
#else
#define A(x) ()
#define A0() ()
#define A1(t1, n1) (n1) t1 n1;
#define A2(t1, n1, t2, n2) (n1, n2) t1 n1; t2 n2;
#define A3(t1, n1, t2, n2, t3, n3) (n1, n2, n3) t1 n1; t2 n2; t3 n3;
#define A4(t1,n1,t2,n2,t3,n3,t4,n4)\
      (n1,n2,n3,n4) t1 n1; t2 n2; t3 n3; t4 n4;
#define A5(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5)\
      (n1,n2,n3,n4,n5) t1 n1;t2 n2;t3 n3;t4 n4;t5 n5;
#define A6(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6)\
      (n1,n2,n3,n4,n5,n6) t1 n1;t2 n2;t3 n3;t4 n4;t5 n5;t6 n6;
#define A7(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,t7,n7)\
      (n1,n2,n3,n4,n5,n6,n7) t1 n1;t2 n2;t3 n3;t4 n4;t5 n5;t6 n6;t7 n7;
#define A8(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,t7,n7,t8,n8)\
      (n1,n2,n3,n4,n5,n6,n7) t1 n1;t2 n2;t3 n3;t4 n4;t5 n5;t6 n6;t7 n7;t8 n8
#endif

/* variable arguments in PROTOTYPE */
#if SUPPORTS_STDARG
#define AV(x) x
#define VA_START(ap,var) va_start(ap,var)
#else
#define AV(x) ()
#define VA_START(ap,var) va_start(ap)
#endif

/* variable argument list in FUNCTION DECLARATION */
#if !ehSUNC && SUPPORTS_STDARG
#define A1V(t1, n1, d) (t1 n1, ...)
#define A2V(t1, n1, t2, n2, d) (t1 n1, t2 n2, ...)
#define A3V(t1, n1, t2, n2, t3, n3, d) (t1 n1, t2 n2, t3 n3, ...)
#define A4V(t1,n1,t2,n2,t3,n3,t4,n4,d)\
      (t1 n1,t2 n2,t3 n3, t4 n4, ...)
#define A5V(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,d)\
      (t1 n1,t2 n2,t3 n3,t4 n4,t5 n5,...)
#define A6V(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,d)\
      (t1 n1,t2 n2,t3 n3,t4 n4,t5 n5,t6 n6,...)
#define A7V(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,t7,n7,d)\
      (t1 n1,t2 n2,t3 n3,t4 n4,t5 n5,t6 n6,t7 n7,...)
#define A8V(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,t7,n7,t8,n8,d)\
      (t1 n1,t2 n2,t3 n3,t4 n4,t5 n5,t6 n6,t7 n7,t8 n8,...)
#define A9V(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,t7,n7,t8,n8,t9,n9,d)\
      (t1 n1,t2 n2,t3 n3,t4 n4,t5 n5,t6 n6,t7 n7,t8 n8,t9 n9,...)
#else
#define A1V(t1, n1, d) (n1, va_alist) t1 n1; va_dcl
#define A2V(t1, n1, t2, n2, d) (n1, n2, va_alist) t1 n1; t2 n2; va_dcl
#define A3V(t1, n1, t2, n2, t3, n3, d)\
      (n1, n2, n3, va_alist) t1 n1; t2 n2; t3 n3; va_dcl
#define A4V(t1,n1,t2,n2,t3,n3,t4,n4,d)\
      (n1,n2,n3,n4, va_alist) t1 n1;t2 n2;t3 n3;t4 n4; va_dcl
#define A5V(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,d)\
      (n1,n2,n3,n4,n5, va_alist) t1 n1;t2 n2;t3 n3;t4 n4;t5 n5; va_dcl
#define A6V(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,d)\
      (n1,n2,n3,n4,n5,n6, va_alist)\
      t1 n1;t2 n2;t3 n3;t4 n4;t5 n5;t6 n6; va_dcl
#define A7V(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,t7,n7,d)\
      (n1,n2,n3,n4,n5,n6,n7,va_alist)\
      t1 n1;t2 n2;t3 n3;t4 n4;t5 n5;t6 n6;t7 n7; va_dcl
#define A8V(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,t7,n7,t8,n8,d)\
      (n1,n2,n3,n4,n5,n6,n7,n8,va_alist)\
      t1 n1;t2 n2;t3 n3;t4 n4;t5 n5;t6 n6;t7 n7;t8 n8;va_dcl
#define A9V(t1,n1,t2,n2,t3,n3,t4,n4,t5,n5,t6,n6,t7,n7,t8,n8,t9,n9,d)\
      (n1,n2,n3,n4,n5,n6,n7,n8,n9,va_alist)\
      t1 n1;t2 n2;t3 n3;t4 n4;t5 n5;t6 n6;t7 n7;t8 n8;t9 n9;va_dcl
#endif

#if ehSUNOS /* prototypes for functions not prototyped in distributed headers */
int    atoi     A((const char *nptr));
void   bcopy    A((const char *src, char *dst, int length));
int    exit     A((int status));
int    fclose   A((FILE *stream));
int    fflush   A((FILE *stream));
int    fprintf  A((FILE *stream, const char *format, ...));
size_t fread    A((void *ptr, size_t size, size_t nmemb, FILE *stream));
size_t fwrite   A((const void*ptr,size_t size,size_t nmemb,FILE*strm));
int    fscanf   A((FILE *stream, const char *format, ...));
int    fputs    A((const char *s, FILE *stream));
int    isatty   A((int fildes));
char*  memcpy   A((void *s1, const void *s2, size_t n));
void * memmove  A((void *s1, const void *s2, size_t n));
char*  memset   A((void *s, int c, size_t n));
void   perror   A((const char *s));
int    printf   A((const char *format, ...));
int    sscanf   A((char *s, const char *format, ...));
time_t time     A((time_t *timeptr));
int    vfprintf A((FILE *stream, const char *format, va_list arg));
#endif

/*** make sure a useable bool type exists ***/

#ifdef HAVE_CONFIG_H 
/* if ./configure was used, bool would be defined in ehconfig.h */

#ifdef HAVE_BUILTIN_BOOL
/* use it! */
#else /* not defined HAVE_BUILTIN_BOOL */
/* then configure should have defined it on the command line */
#ifndef bool /* just in case... */
#define bool char  /* if not, define it here */
#endif /* bool */
#endif /* not defined HAVE_BUILTIN_BOOL */

#else  /* not defined HAVE_CONFIG */
/* if configure was not used, check the environment and act accordingly */

#if defined(__cplusplus) && ehGNUC
/* have built-in bool */
#else /* need to define bool */

/***
   When compiling on the BeBox I get different results
   depending if I use the IDE or not.  If the IDE is used
   bool seems to be defined.  This may have to do with the
   'BeHeaders' prefix defined in the IDE settings.  If the 
   command line is used bool needs to be defined here.
***/
#if ehMWCC && !defined(_SUPPORT_DEFS_H) /* Be typedefs bool in here */
typedef char bool; /* if not, define it here */
#elif !defined(bool) /* not ehBEOS or not  _SUPPORT_DEFS_H */
#define bool char /* if not, define it here */
#endif
#endif /* ! defined(__cplusplus) && ehGNUC */

#endif /* not defined HAVE_CONFIG_H */

#ifndef HAVE_BUILTIN_TRUE
#ifndef true
#define true 1
#endif /* true */
#endif /* HAVE_BUILTIN_TRUE */

#ifndef HAVE_BUILTIN_FALSE
#ifndef false
#define false 0
#endif /* false */
#endif /* HAVE_BUILTIN_FALSE */

/*** other abreviations ***/

/* generic NULL pointer */
#ifndef NULL
#define NULL  0
#endif

/* ASCII NUL character */
#ifndef NUL
#define NUL ((char)0)
#endif

/* support for repeat-until construct */
#ifndef repeat
#define repeat   do
#endif

#ifndef until
#define until(x) while(!(x))
#endif

/* some abreviations */
#define SZCHR sizeof(char)
#define SZINT sizeof(int)
#define SZLNG sizeof(long)
#define SZARR(a) (sizeof(a)/sizeof(*a)) /* # elements in an array */

/* minumum */
#ifndef MIN
#define MIN(i,j) ((i) < (j) ? i : j)
#endif

/* maximum */
#ifndef MAX
#define MAX(i,j) ((i) < (j) ? j : i)
#endif

/* absolute value */
#ifndef ABS
#define ABS(i)   ((i) < 0 ? -(i) : i)
#endif

/* boolean inclusive check if variable is in range */
#define IN_RANGE(x, lo, hi)  ((lo) <= (x) && (x) <= (hi))

/* free memory and set pointer to NULL */
#ifndef EHFREE
#define EHFREE(p) free(p),p=NULL
#endif

/* boundary alignment requirements on given data type */
static ALIGNTYP _at = 0;   /* alignment type for this compiler */
#define BOUNDOF(typ)\
      ((char *)&((struct{char C;typ T;}*)&_at)->T-(char *)&_at)

/* report offsets of structure members */
#if !defined(offsetof) && !defined(__STDC__) && !defined(__cplusplus)
#define offsetof(stc,m) ((char *)&((stc *)&_at)->m - (char *)&_at)
#endif

/****   data types abbreviations   ****/

#ifndef ulong
#define ulong unsigned long
#endif

#ifndef uint
#define uint unsigned
#endif

#ifndef ushort
#define ushort unsigned short
#endif

#ifndef schar
#define schar signed char
#endif

#ifndef uchar
#define uchar unsigned char
#endif

#ifndef word
#define word unsigned short
#endif

#if !defined(byte) && !(ehSASC && defined(_DOS_H))
#define byte unsigned char
#endif

#ifndef uint
#define uint unsigned
#endif

#define NOREF(v) (v)=(v) /* to suppress 'unreferenced-variable' messages */

#define istext(c) ( isascii(c) && !iscntrl(c) )

/* allocate memory for a new structure */
#define NEW(stc) (stc *)calloc(sizeof(stc), 1)

/*** global variable declarations ***/

typedef void (*pfv_v)(void);

typedef struct
{
#define EH_SZ_PREFIX 4               /* 1-n character prefix */
   char    acPrefix[EH_SZ_PREFIX+1]; /* error prefix */
   short   hCode;                    /* error code */
} ER; /* error-handler error */

typedef struct
{
   char         *pcProgName;   /* program name */
   byte          yIniState;    /* Error Handler initialization state */
   FILE         *pFilOut;      /* output that will be seen by user */
   int           iLoHi;        /* CPU byte ordering */
   ER            Er;           /* error source and number */
   ulong         ulCtrlCIni;   /* initial SIMCTRLC value */
   ulong         ulCtrlCVal;   /* incremented SIMCTRLC value */
   volatile byte yIntSig;      /* interrupt (Ctrl-C) sig has occurred */
   byte          yNoCtrlC;     /* disable Ctrl-C checking */
   byte          yNoBounds;    /* disable bounds checking */
   size_t        szOsTerm;     /* # functions to call at ehOsTerm() */
   pfv_v        *ppfvOsTerm;   /* array of function pointers */
} EH;
extern EH Eh;  /* global variables for eh library */

/* disable Control-C Checking */
#define EH_CTRLC_CHK_OFF() (Eh.yNoCtrlC = 1)
#define EH_CTRLC_CHK_ON()  (Eh.yNoCtrlC = 1)

/* disable Bounds Checking */
#define EH_BNDS_CHK_OFF() (Eh.yNoBounds = 1)
#define EH_BNDS_CHK_ON()  (Eh.yNoBounds = 0)

/*** Function entry/exit tracing support ***/

/***
The symbol __CHAN__ is the variable which is accessed at runtime to
determine the channel to send error messages.  
***/

/*** channel names ***/
#define EH_CHAN_UNIT  0   /* 0 = unit development */
#define EH_CHAN_ALPHA 1   /* 1 = alpha  */
#define EH_CHAN_BETA  2   /* 2 = beta */
#define EH_CHAN_MAX   2   /* highest channel number */

/*** channel setting ***/
#ifndef EH_CHAN     
#define EH_CHAN EH_CHAN_UNIT /* default to unit channel */
#else
#if EH_CHAN > EH_CHAN_MAX
#undef EH_CHAN
#define EH_CHAN EH_CHAN_MAX
#endif
#endif
static int __CHAN__ = EH_CHAN;

#define FUN_CHAN(ch) static int __CHAN__ = (ch)
#define EHCHAN(ch) static int __CHAN__ = (ch)

/*** volume levels ***/
#define EH_VOL_ERR  0 /* error messages */
#define EH_VOL_WRN  1 /* warning messages */
#define EH_VOL_REM  2 /* remark messages */
#define EH_VOL_DBG  3 /* debug messages */

/* module file name definition macro: with & without ANSI __FILE__ */
#ifndef __FILE__
#define EHFILENAME(fn) static const char *__FILE__ = #fn
#else
/* we need *some* definition here to satisfy some compilers */
#define EHFILENAME(fn) typedef char _eh_dummy_ 
#endif

/*** structure to remember state of parent ***/

#define EHSTATE struct _eh_state_
EHSTATE
{
   const char *pcFile;   /* filename of func these settings are for */
   const char *pcFunc;   /* function these settings are for */
   int         iLine;    /* line this func starts on (if func is null)*/
   byte        yCatch;   /* current function is a catcher */
   byte        yChain;   /* chaining has been requested */
   byte        yCtrlC;   /* Control-C (interrupt signal) has occurred */
   byte        yInhibit; /* longjmp()ing is inhibited */
   byte        yShield;  /* shielding has been requested */
   signed char scError;  /* an error occurred in the current function */
   short       hLev;     /* catcher's nesting level */
   EHSTATE    *pEhSv;    /* pointer to current EHSTATE variable */
#ifndef ehEXCEPTIONHANDLING
   jmp_buf     JmpBuf;   /* longjmp target: closest catcher */
#endif
};
extern EHSTATE EhSt; /* current error handler state */

/*** some public function definitions ***/

extern void (*pfvEhIntInit)A0(); /* internal initialization vector */
extern void (*pfvEhExtInit)A0(); /* external initialization vector */

void  ehAtExit      A((pfv_v pfvTermFunc));
void  ehOsTerm      A((void));

#if defined(EH_NO_MEMORY_TRACKING) || (EH_CHAN == EH_CHAN_MAX)
#define CHKOVR 0 /* don't check for memory overwrites */
#else
#define CHKOVR 1 /* check for memory overwrites */
#endif

#define EHCTRLC_DEFAULT ((ulong)~0L)
#ifdef EHSIMCTRLC /* compiler-line request for CTRL-C simulation */
#define SIMCTRLC() \
do{if(Eh.ulCtrlCVal==-1L)Eh.ulCtrlCVal=Eh.ulCtrlCIni;if(--Eh.ulCtrlCVal==0L)ehSigInt(SIGINT);}while(0)
#else
#define SIMCTRLC()
#endif

#define EHFORCECTRLC() ehSigInt(SIGINT)


#define EHBREAK goto _EhEndCatch

/*** define EhEndCatch macros ***/

#if ehBC
#define EHCALLBACK _USERENTRY
#else
#define EHCALLBACK
#endif

void ehSigInstall A((void));   /* install SIGINT handler */
void ehSigUninstall A((void)); /* uninstall SIGINT handler */
void EHCALLBACK ehSigInt A((int iSig)); /* to match prototype */

#define EHSTOPCHAIN() (EhSt.yCtrlC=EhSt.yChain=0) /* stop longjmp chain */
#define EHCLEAR()     (Eh.Er.acPrefix[0]='\0',Eh.Er.hCode=0)/* clear error */

#define EHSHIELDON()  (EhSt.yShield=1)
#define EHSHIELDOFF() (EhSt.yShield=0)
#define EHINHIBIT()   (EhSt.yInhibit=1)

/*** these macros can only be used in the exit sequence ***/

#define ehError()     (EhSt.scError != 0)    /* error (EHTHROW,EHERROR) occurred in CURRENT function */
#define ehFatal()     (EhSt.scError == -1)   /* error (EHTHROW,EHERROR) occurred in CURRENT function */
#define ehAnyError()  (Eh.Er.u.hNo != 0)  /* any error has occurred in this process */
#define ehJmpError()  (iJmpVal!=0)          /* error inherited from longjmp */
#define ehCtrlC()     (EhSt.yCtrlC!=0)      /* user pressed control C */

#define OFFSET_WIDTH 23  /* # bits in file offset part of error message code */

#if ehBC
#define ehErrorIs(e0,e1)  ( ehUnHash(e1)  == Eh.Er )
#else
bool ehErrorIs A((ulong ulOffset, ulong ulHash)); /* c.f. ehercode.c */
#endif

/* check for control-c within a function */
#define EHCTRLC() do{\
   ehChkCtrlC();\
   if (EhSt.yCtrlC)\
   {\
      EhSt.scError = -1;\
      EHBREAK;\
   } } while(0)
#define EHCTRLC_RETURN(val) do{\
   ehChkCtrlC();\
   if (EhSt.yCtrlC)\
   {\
      EhSt.scError = -1;\
      EHRETURN(val);\
   } } while(0)
#define EHCTRLC_LEAVE() do{\
   ehChkCtrlC();\
   if (EhSt.yCtrlC)\
   {\
      EhSt.scError = -1;\
      EHLEAVE();\
   } } while(0)
#define EH_CTRLC 1  /* for use with ehErrVal() */

/*** error processing macros ***/

void ehFunEnt    A((int,const char*,const char*,const int,int,EHSTATE*));
int  ehCatch     A((EHSTATE*pEhSv));
void ehFunRet    A((int,const char*,const char*,const int,int));
void ehFatalJmp  A((void));
void ehExitJmp   A((void));
void ehPostJmp   A((const char*));
void ehChkCtrlC  A((void));
int  ehSetFun    A((int iChan, int iFunBit));
int  ehSetVol    A((int iChan, int iFunBit));
int  ehGetVol    A((int iChan));
void ehHexDmp    A((FILE *pFileOut, const char *pcData, int iNum));
void ehLogTxt    A((const char *pcLogTxt));
void ehPError    A((const char*,const char*,int,schar,ulong,ulong));
void ehPgmEnt    A((void));
void ehPgmExt    A((int iChan));
void*ehDynArr    A((void **, size_t*, const size_t));
uint ehRoundUp   A((unsigned int uiValue));
void ehCallStack A((FILE*));

extern short *phEhChnVol;
#define EH_BITS_PER_CHAN 5     /* total # bits per channel */
#define EH_NUMCHAN       3

#if 1 /* 1 = macro, 0 = function */
#define ehIndent(pFile)\
   fprintf(pFile, "%*.*s", EhSt.hLev * 3, EhSt.hLev * 3, ""),fflush(pFile)
#else
void ehIndent  A((FILE *pFile));
#endif

#if 1 /* 1 = macro, 0 = function */
#define ehGetOut()\
    ( (int)(*phEhChnVol >> (EH_BITS_PER_CHAN * EH_NUMCHAN)) & 1 ) 
#else
int ehGetOut A((void)); /* import from .c file */
#endif

void ehLogFun AV((const char *pcTxt,int iChn,int iVol,const char *pcFil,
                  const char *pcFunc,int iLin, ...));


#define EHCHAIN                do{EhSt.yChain=1;EHBREAK;}while(0)
#define EHCHAIN_RETURN(RetVal) do{EhSt.yChain=1;EHRETURN(RetVal);}while(0)
#define EHCHAIN_LEAVE()        do{EhSt.yChain=1;EHLEAVE();}while(0)

#define EH_IS_ERROR ((schar)1)
#define EH_IS_FATAL ((schar)-1)

/*** #if (ehGNUC & ehNIX) | ehBC ***/

#if ehBC && (ehBCVER < 310)

#define _FATAL(e0,e1) \
   ehPError(__FILE__, _EHFUNC_, __LINE__, (schar)-1, e0,(ER)e1)

#define EHTHROW(e0,e1)       do{_FATAL(e0,e1);EHBREAK;}while(0)
#define EHTHROW_RETURN(e0,e1,RetVal) do{_FATAL(e0,e1);EHRETURN(RetVal);}while(0)
#define EHTHROW_LEAVE(e0,e1)         do{_FATAL(e0,e1);EHLEAVE();}while(0)

#define _ERROR(e0,e1) \
   ehPError(__FILE__, _EHFUNC_, __LINE__, 1, e0,(ER)e1)

#define EHERROR(e0,e1)               do{_ERROR(e0,e1);EHBREAK;}while(0)
#define EHERROR_RETURN(e0,e1,RetVal) do{_ERROR(e0,e1);EHRETURN(RetVal);}while(0)
#define EHERROR_LEAVE(e0,e1)         do{_ERROR(e0,e1);EHLEAVE();}while(0)

#else /* conventional definition */

#define EHTHROW(scError)\
 do{ehPError(__FILE__,_EHFUNC_,__LINE__,(schar)-1,scError);EHBREAK;}while(0)
        
#define EHTHROW_RETURN(scError,RetVal)\
 do{ehPError(__FILE__,_EHFUNC_,__LINE__,(schar)-1,scError);EHRETURN(RetVal);}while(0)
        
#define EHTHROW_LEAVE(scError)\
 do{ehPError(__FILE__,_EHFUNC_,__LINE__,(schar)-1,scError);EHLEAVE();}while(0)


#define EHERROR(scError)\
 do{ehPError(__FILE__,_EHFUNC_,__LINE__,1,scError);EHBREAK;}while(0)
        
#define EHERROR_RETURN(scError,RetVal)\
 do{ehPError(__FILE__,_EHFUNC_,__LINE__,1,scError);EHRETURN(RetVal);}while(0)
        
#define EHERROR_LEAVE(scError)\
 do{ehPError(__FILE__,_EHFUNC_,__LINE__,1,scError);EHLEAVE();}while(0)


#endif

#define EHSTDOUT (Eh.pFilOut)
#define EHSTDERR (ehGetOut() == 1 ? stdout : stderr)

#define _ASSERT(EX) \
   ehLogTxt("\n");\
   ehIndent(EHSTDERR);\
   fprintf(EHSTDERR,\
         "EHASSERT..(%s) failed: %s <%s %s() %d>\n",\
         #EX, Eh.pcProgName, __FILE__, _EHFUNC_, __LINE__);\
   fflush(EHSTDERR)
   
#define EHASSERT(EX)               if (!(EX))do{_ASSERT(EX);EHTHROW(EH_ASSERT);}while(0)
#define EHASSERT_RETURN(EX,RetVal) if (!(EX))do{_ASSERT(EX);EHTHROW_RETURN(EH_ASSERT,RetVal);}while(0)
#define EHASSERT_LEAVE(EX)         if (!(EX))do{_ASSERT(EX);EHTHROW_LEAVE(EH_ASSERT);}while(0)

#define EH_SZBUF 81

/*** EHERR macros ***/

#define EHERRX(s) \
   { s; }

#define EHERR(s) \
   ehLogFun(s, __CHAN__, EH_VOL_ERR, __FILE__, _EHFUNC_, __LINE__)

#define EHERR1(s, a1) \
   ehLogFun(s,__CHAN__, EH_VOL_ERR, __FILE__, _EHFUNC_, __LINE__,a1)

#define EHERR2(s, a1, a2) \
   ehLogFun(s,__CHAN__, EH_VOL_ERR, __FILE__, _EHFUNC_, __LINE__,a1,a2)

#define EHERR3(s, a1, a2, a3) \
   ehLogFun(s,__CHAN__, EH_VOL_ERR, __FILE__, _EHFUNC_, __LINE__,a1,a2,a3)

/*** EHWRN macros ***/

#define EHWRNX(s) \
if (ehGetVol(__CHAN__) >= EH_VOL_WRN)\
{\
   s;\
}

#define EHWRN(s) \
   ehLogFun(s,__CHAN__, EH_VOL_WRN, __FILE__, _EHFUNC_, __LINE__)

#define EHWRN1(s, a1) \
   ehLogFun(s,__CHAN__, EH_VOL_WRN, __FILE__, _EHFUNC_, __LINE__,a1)

#define EHWRN2(s, a1, a2) \
   ehLogFun(s,__CHAN__, EH_VOL_WRN, __FILE__, _EHFUNC_, __LINE__,a1,a2)

#define EHWRN3(s, a1, a2, a3) \
   ehLogFun(s,__CHAN__, EH_VOL_WRN, __FILE__, _EHFUNC_, __LINE__,a1,a2,a3)

/*** EHREM macros ***/

#define EHREMX(s) \
if (ehGetVol(__CHAN__) >= EH_VOL_REM)\
{\
   s;\
}

#define EHREM(s) \
   ehLogFun(s, __CHAN__, EH_VOL_REM, __FILE__, _EHFUNC_, __LINE__)

#define EHREM1(s, a1) \
   ehLogFun(s,__CHAN__, EH_VOL_REM, __FILE__, _EHFUNC_, __LINE__,a1)

#define EHREM2(s, a1, a2) \
   ehLogFun(s,__CHAN__, EH_VOL_REM, __FILE__, _EHFUNC_, __LINE__,a1,a2)

#define EHREM3(s, a1, a2, a3) \
   ehLogFun(s,__CHAN__, EH_VOL_REM, __FILE__, _EHFUNC_, __LINE__,a1,a2,a3)

/*** EHDBG macros ***/

#define EHDBGX(s) \
if (ehGetVol(__CHAN__) >= EH_VOL_DBG)\
{\
   s;\
}

#define EHDBG(s) \
   ehLogFun(s, __CHAN__, EH_VOL_DBG, __FILE__, _EHFUNC_, __LINE__)

#define EHDBG1(s, a1) \
   ehLogFun(s,__CHAN__, EH_VOL_DBG, __FILE__, _EHFUNC_, __LINE__,a1)

#define EHDBG2(s, a1, a2) \
   ehLogFun(s,__CHAN__, EH_VOL_DBG, __FILE__, _EHFUNC_, __LINE__,a1,a2)

#define EHDBG3(s, a1, a2, a3) \
   ehLogFun(s,__CHAN__, EH_VOL_DBG, __FILE__, _EHFUNC_, __LINE__,a1,a2,a3)


#ifndef ehEXCEPTIONHANDLING /*** C model exception handling ***/


/* _FUN_TRACE() is used by both FUN_TRACE() and PGM_TRACE() */

#ifdef EH_SUPPLY_FUNCTION_NAME
#define _FUN_TRACE(pcFunc)\
   SIMCTRLC();\
   ehFunEnt(__CHAN__,__FILE__,pcFunc,__LINE__,CHKOVR,&EhSv)
#else  /* don't need to EH_SUPPLY_FUNCTION_NAME */
#define _FUN_TRACE()\
   SIMCTRLC();\
   ehFunEnt(__CHAN__,__FILE__,__FUNC__,__LINE__,CHKOVR,&EhSv)
#endif /* don't need to EH_SUPPLY_FUNCTION_NAME */

/* FUN_TRACE() is the function tracing entry point */

#ifdef EH_SUPPLY_FUNCTION_NAME
#define FUN_TRACE(pcFunc)\
      const char *volatile _EHFUNC_ = pcFunc;\
      EHSTATE EhSv;\
      _FUN_TRACE(_EHFUNC_)
#else  /* don't need to EH_SUPPLY_FUNCTION_NAME */
#define FUN_TRACE()\
      const char *volatile _EHFUNC_ = __FUNC__;\
      EHSTATE EhSv;\
      _FUN_TRACE(_EHFUNC_)
#endif /* don't need to EH_SUPPLY_FUNCTION_NAME */

/* EHLEAVE() is required for functions returning void */

#define EHLEAVE()\
 do{ehFunRet(__CHAN__,__FILE__,_EHFUNC_,__LINE__,CHKOVR);return;}while(0)

/* EHRETURN() is required for functions returning a value */

#define EHRETURN(RetVal)\
   return (ehFunRet(__CHAN__,__FILE__,_EHFUNC_,__LINE__,CHKOVR),RetVal)

#define _FUN_CATCH()\
 if (ehCatch(&EhSv)) EHBREAK;\
 if (setjmp(EhSt.JmpBuf))\
 {\
  ehPostJmp(_EHFUNC_);\
 }\
 else\
 {

#ifdef EH_SUPPLY_FUNCTION_NAME
#define FUN_CATCH(pcFunc)\
 FUN_TRACE(pcFunc);\
 _FUN_CATCH()
#else  /* don't need to EH_SUPPLY_FUNCTION_NAME */
#define FUN_CATCH()\
 FUN_TRACE();\
 _FUN_CATCH()
#endif /* don't need to EH_SUPPLY_FUNCTION_NAME */

#define ExitSequence \
}_ExitSequence:\
EHINHIBIT();\
__ExitSequence

#define EhCatchPoint \
}_ExitSequence:\
EHINHIBIT();\
__ExitSequence

/* _PGM_TRACE() is the core definition for PGM_TRACE() */

#define _PGM_TRACE() \
   EHSTATE EhSv;\
   Eh.pcProgName = (char *)argv[0];argc=argc;\
   ehPgmEnt()

/* PGM_TRACE() performs function tracing initilization in main */

#ifndef _EHFUNC_
/* we use an explicit "main" here to later check if we are in main() */
#define PGM_TRACE()\
 const char *_EHFUNC_="main";\
 _PGM_TRACE();\
 _FUN_CATCH();\
 _FUN_TRACE(_EHFUNC_)
#else
#define PGM_TRACE()\
 _PGM_TRACE();\
 _FUN_CATCH();\
 _FUN_TRACE(_EHFUNC_)
#endif

/* PGM_RETURN() is required: you should *always* return a value to the OS */

#define PGM_RETURN(RetVal)\
   return(\
      ehOsTerm(),\
      ehFunRet(__CHAN__,__FILE__,_EHFUNC_,__LINE__,CHKOVR),\
      ehPgmExt(__CHAN__),\
      RetVal)


#else /*** ehEXCEPTIONHANDLING *** C++ model exception handling ***/

class EHMEFIRST
{
public:
   EHMEFIRST();   // wrapper
   ~EHMEFIRST();  // object
};

// eXception Handler support

extern char cEhFunTrc; // internal function tracing
extern char cEhPgmTrc; // internal program  tracing

class EHFUNTRACE
{
public:
   EHFUNTRACE();
   EHFUNTRACE(const char*pcFile, const char*pcFunc, int iLine, int iChan,
         bool chkOvrwrt);
   void ft_Args(int *piArgc, char **ppcArgv);
   void ft_Init(const char*pcFile, const char*pcFunc, int iLine, int iChan,
         bool chkOvrwrt, int * = 0, char ** = 0);
   void ft_Catch();
   void ft_LineExit(const int);
   ~EHFUNTRACE();
private:
   bool         ft_Destroyed; // work-around for compiler bug
   void         ft_PgmEnt(int *piArgc, char **ppcArgv); // program entry
   void         ft_PgmExt(); // program exit
   const char  *ft_pcFile;
   const char  *ft_pcFunc;
   int          ft_iLine;
   int          ft_iLineExit;
   int          ft_iChan;
   bool         ft_chkOvrwrt;
   EHSTATE     ft_EhSt;
};

// replace the eh version of function tracing macros with xh version

const bool chkOvrwrt = (bool)CHKOVR;

#ifndef _EHFUNC_
#ifdef EH_SUPPLY_FUNCTION_NAME
#define EHTRACE(sFunc)\
 SIMCTRLC();\
 const char *volatile _EHFUNC_ = #sFunc;\
 EHFUNTRACE ehFunTrace(__FILE__,_EHFUNC_,(const int)__LINE__,__CHAN__,chkOvrwrt)
#else /* don't need to EH_SUPPLY_FUNCTION_NAME */
#define EHTRACE()\
 SIMCTRLC();\
 const char *volatile _EHFUNC_ = __FUNC__;\
 EHFUNTRACE ehFunTrace(__FILE__,_EHFUNC_,(const int)__LINE__,__CHAN__,chkOvrwrt)
#endif /* don't need to EH_SUPPLY_FUNCTION_NAME */
#else /* _EHFUNC_ is defined (as NULL, for no-function-tracing support) */
#ifdef EH_SUPPLY_FUNCTION_NAME
#define EHTRACE(sFunc)\
 SIMCTRLC();\
 EHFUNTRACE ehFunTrace(__FILE__,#sFunc,(const int)__LINE__,__CHAN__,chkOvrwrt)
#else /* don't need to EH_SUPPLY_FUNCTION_NAME */
#define EHTRACE()\
 SIMCTRLC();\
 EHFUNTRACE ehFunTrace(__FILE__,__FUNC__,(const int)__LINE__,__CHAN__,chkOvrwrt)
#endif /* don't need to EH_SUPPLY_FUNCTION_NAME */
#endif /* _EHFUNC_ is defined (as NULL) */

#define EHLEAVE() do{ehFunTrace.ft_LineExit(__LINE__);return;}while(0)

#define EHRETURN(val) do{ehFunTrace.ft_LineExit(__LINE__);return(val);}while(0)

#ifdef EH_SUPPLY_FUNCTION_NAME
#define EHTRY(sFunc)\
 EHTRACE(sFunc);\
 ehFunTrace.ft_Catch();\
 do { try
#else /* don't need to EH_SUPPLY_FUNCTION_NAME */
#define EHTRY()\
 EHTRACE();\
 ehFunTrace.ft_Catch();\
 do { try
#endif /* don't need to EH_SUPPLY_FUNCTION_NAME */


void xhCaught(const char *pcFunc);

#ifndef _EHFUNC_
#define EHMAIN(ac,av)\
 Eh.pcProgName = av[0];\
 const char *volatile _EHFUNC_ = "main";\
 EHFUNTRACE ehFunTrace;\
 do {\
 ehFunTrace.ft_Catch();\
 try {\
 ehFunTrace.ft_Init(__FILE__,_EHFUNC_,__LINE__,__CHAN__,chkOvrwrt, &ac, av);
#else /* _EHFUNC_ is defined (as NULL) */
#define EHMAIN(ac,av)\
 Eh.pcProgName = av[0];\
 EHFUNTRACE ehFunTrace;\
 do {\
 ehFunTrace.ft_Catch();\
 try {\
 ehFunTrace.ft_Init(__FILE__,_EHFUNC_,__LINE__,__CHAN__,chkOvrwrt, &ac, av);
#endif

#define EHCATCH \
 catch(...){xhCaught(_EHFUNC_);}}while(0);_EhEndCatch:EHINHIBIT()

#define EHENDMAIN \
 }catch(...){xhCaught(_EHFUNC_);}}while(0);_EhEndCatch:EHINHIBIT()

#define EHDONE(val) do{ehFunTrace.ft_LineExit(__LINE__);return(val);}while(0)

#endif /* ehEXCEPTIONHANDLING */


/*** former es.h -- Error-handling Support library header ***/

/*** support for ehgtargs.c (command-line argument processing) ***/

#define EH_GA_MAND 0x8000 /* for mandatory paramters */
typedef enum
{
   EH_GA_MESSAGE = 1, /* display given text before processing command line */
   EH_GA_USAGE,       /* description of command line usage */
   EH_GA_BOOL,        /* the mere presence of switch sets BOOL var to 1 */
   EH_GA_CBOOL,       /* the mere presence of switch sets CHAR var to 1 */
   EH_GA_HBOOL,       /* the mere presence of switch sets SHORT var to 1 */
   EH_GA_IBOOL,       /* the mere presence of switch sets INT var to 1 */
   EH_GA_CHAR,        /* place next character in char @ pvValue */
   EH_GA_ICHAR,       /* place next character in int @ pvValue */
   EH_GA_PCHAR,       /* place address of string at pointer @ pvValue */
   EH_GA_ACHAR,       /* copy text to buffer pointed to by pvValue */
   EH_GA_SHORT,       /* place next numeric string in short @ pvValue */
   EH_GA_INT,         /* place next numeric value in int @ pvValue */
   EH_GA_SIZE_T,      /* place next numeric value in size_t @ pvValue */
   EH_GA_LONG,        /* place next numeric value in long @ pvValue */
   EH_GA_FLOAT,       /* place next numeralue in float @ pvValue */
   EH_GA_DOUBLE       /* place next numeric value in double @ pvValue */
} EHGATYPE;

void  ehGtArgs  AV((volatile int * piArgc, char ***pppcArgv, ...));


/*** support for ehlist.c (linked list) ***/

#define EHNODE struct ehMinNode

EHNODE
{
   EHNODE *pEhNodeSucc;
   EHNODE *pEhNodePred;
};

typedef struct ehMinList
{
   EHNODE *pEhNodeHead;
   EHNODE *pEhNodeTail;
   EHNODE *pEhNodeTailPred;
} EHLIST;

bool    ehAddHead     A((EHLIST *pEhList, EHNODE *pEhNode));
bool    ehAddTail     A((EHLIST *pEhList, EHNODE *pEhNode));
void    ehInsert      A((EHLIST *pEhList, EHNODE *pEhNodeNew, EHNODE *pEhNodePred));
bool    ehListIsEmpty A((EHLIST *pEhList));
void    ehNewList     A((EHLIST *pEhList));
EHNODE *ehRemHead     A((EHLIST *pEhList));
EHNODE *ehInqHead     A((EHLIST *pEhList));
bool    ehRemove      A((EHNODE *pEhNode));
EHNODE *ehRemTail     A((EHLIST *pEhList));
EHNODE *ehLocTail     A((EHLIST *pEhList));
#define ehLocPred(pEhNode) ((pEhNode)->pEhNodePred)

/* linked list (named, prioritized) support */
void    ehEnqueue     A((EHLIST *pEhList, EHNODE *pEhNode));
EHNODE *ehFindName    A((EHLIST *pEhList, byte *pbName));

/* traversing a list */
#define ehListHead(pEhList) ((pEhList)->pEhNodeHead)
#define ehListEnd(pEhNode)  ((pEhNode)->pEhNodeSucc == 0)
#define ehListNext(pEhNode) ((pEhNode)->pEhNodeSucc)


/*** support for ehavltre.c (avl tree) ***/

extern char cEhAvlDmp;

typedef void (*pfv_pv)      (const void*);            /* ptr to func ret void */
typedef void (*pfv_pFilepv) (FILE*, const void*);     /* ptr to func ret void */
typedef int  (*pfi_pv)      (const void*);             /* ptr to func ret int */
typedef int  (*pfi_ppv)     (const void**);            /* ptr to func ret int */
typedef int  (*pfi_pvpv)    (const void*,const void*); /* ptr to func ret int */

typedef enum 
{
   EH_TOP_HVY = -1,
   EH_BALANCED,
   EH_BTM_HVY
} EHBALSTAT;       /* current balance status of a node */

#define EHAVLNODE struct _ehavlnode_
EHAVLNODE
{
   EHAVLNODE *pEhAvlNodeSelf;  /* self-referent address */
   EHAVLNODE *pEhAvlNodeUp;    /* left sub-tree */
   EHAVLNODE *pEhAvlNodeDn;    /* right sub-tree */
   EHBALSTAT  EhBalStat;       /* balance status */
   const void*pvUserData;      /* user data */
};

typedef struct
{
   EHAVLNODE*  pEhAvlNodeTop; /* top node of tree */
   pfi_pvpv    pfiCompare;    /* compare function */
   pfi_ppv     pfiDelete;     /* delete/free function */
   pfv_pFilepv pfvDump;       /* dump user data */
} EHAVL;

typedef void (*pfv_ppEhAvlNode)(EHAVLNODE**);

EHAVL*ehAvlInit     A((pfi_pvpv pfiCompare, pfi_ppv pfiDelete,
                        pfv_pFilepv pfvDump));
void *ehAvlSrch     A((EHAVL *pEhAvl, const void *pvUserData));
bool  ehAvlAdd      A((EHAVL *pEhAvl, const void *pvUserData,
                        pfi_pvpv pfiCompare));
int   ehAvlDel      A((EHAVL *pEhAvl, const void *pvUserData,
                        pfi_pvpv pfiCompare));
void  ehAvlTerm     A((EHAVL *pEhAvl));
int   ehAvlTrav     A((EHAVL*pEhAvl, pfv_ppEhAvlNode pfvAvlNodeFunc,
                        pfi_ppv pfiUserFunc));
void  ehAvlDump     A((EHAVL*pEhAvl, FILE *pFile));
void  ehAvlTreeDump A((FILE*, EHAVL*pEhAvl));

/*** support for ehchkres.c (check resource tracking results) ***/

extern char cEhResDmp;
extern char cEhFunDmp;

typedef struct
{
   bool   isInitialized; /* true if this structure has been initialized */
   size_t szResCount;    /* # resources used */
   ulong  ulResObtained; /* # times resources were obtained */
   ulong  ulResReleased; /* # times resources were released */
   ulong  ulResPeak;     /* # max resources used at one time */
   EHLIST EhList;        /* list header to keep track of allocation order */
   EHAVL *pEhAvlRes;     /* pointer to avl tree tracking resources */
   EHAVL *pEhAvlFun;     /* pointer to avl tree tracking function calls */
} EHRESTRK;

void    ehResInit     A((EHRESTRK *pEhResTrk));
void    ehResTerm     A((EHRESTRK *pEhResTrk));
void    ehResDmpAvl   A((EHAVL* pEhAvl, int iLevel));

/*** support for ehtrkres.c (resource tracking) ***/

typedef ulong (*pful_ulv)(ulong,...); /* ptr to func ret ulong, var args */
typedef void  (*pfv_ulv)(ulong,...);  /* ptr to func ret void, ulong arg */

#define EHRESLIST struct _eh_reslist

/* keep track of multiple CALLs to obtain same RESource */
typedef struct
{
   EHNODE      EhNode;     /* linked list node */
   EHRESLIST  *pEhResList; /* pointer back to parent */
   int         iObtFun;    /* actual resource-obtaining func */
   ulong      *pulArgs;    /* args used when obtaining this resource */
   const char *pcFileO;    /* source file where resource was obtained */
   const char *pcFuncO;    /* function in which resource was obtained */
   int         iLineO;     /* line on which resource was obtained */
   ulong       ulCountO;   /* which time through this statement */
   const char *pcFileR;    /* source file where resource was released */
   const char *pcFuncR;    /* function in which resource was released */
   int         iLineR;     /* line on which resource was released */
   ulong       ulCountR;   /* which time through this statement */
} EHRESCALL, RC;           /* track resource function calls */

extern EHRESTRK ehResTrk; /* internal resource tracking anchor */

/* internally tracked resources */
typedef enum _eh_int_res_
{
   EH_RES_UNKNOWN,  /* un-determined (yet) */
   EH_RES_C_MEM,    /* ANSI C dynamically allocated memory */
   EH_RES_CPP_MEM,  /* ANSI C++ dynamically allocated memory */
   EH_RES_FILESTR,  /* FILE stream pointer */
   EH_RES_MAX       /* number of built-in resource entries */
} EHINTRES;         /* internally tracked resource types */

typedef enum 
{
   EH_RES_NONE,
   EH_RES_RELEASE,
   EH_RES_REOBTAIN,
   EH_RES_OBTAIN
} EHRESWHOM;        /* who most recently operated on the resource */

typedef enum 
{
   EH_RES_UNUSED,
   EH_RES_RELEASED, /* have released the resource */
   EH_RES_OBTAINED  /* have obtained the resource */
} EHRESSTATE;       /* state of resource */

/* contains attributes of each resource in use */
EHRESLIST
{
   int        iType;      /* unique resource type */
   char      *pcResNam;   /* name of resource */
   pfv_ulv    pfvDefRel;  /* ptr to default resource-releasing func */
   pfv_ulv    pfvActRel;  /* ptr to actual  resource-releasing func */
   ulong      ulFlags;    /* define some resource characteristics */
   ulong      ulResHndl;  /* resource handle */
   EHRESWHOM  EhResWhom;  /* who most recently operated on resource */
   EHRESSTATE EhResState; /* state of resource */
   bool       hasProblem; /* problem discovered with resource */
   ushort     uhNumObtn;  /* # RESCALL structs OBTAINED by this res */
   size_t     szRCInUse;  /* # RESCALL structures IN USE */
   size_t     szRCAlloc;  /* # RESCALL structures allocated */
   EHRESCALL *pEhResCall; /* keep track of function calls */
   EHRESLIST *pEhRLOnRel; /* related resource to release 
                             when this one is released */
};

/* line and file where resource obtaining function was called */
typedef struct _eh_res_func_
{
   int   iLine;        /* line where obtaining function was called */
   const char *pcFile; /* file where obtaining function was called */
   ulong ulCount;      /* # times this function has been called */
} EHRESFUNC;

/* EhResList.ulFlags */

#define EH_RES_NUMARGS_MSK 7         /* 0 to 7 arguments are allowed */
#define EH_RES_NUMARGS_WID 3         /* # bits in RES_NUMARGS_MSK */

/* resource returns no value */
#define EH_RES_VOID        (1L << (EH_RES_NUMARGS_WID + 0))
/* resource can be obtained multiple times */
#define EH_RES_ALLOW_DUPS  (1L << (EH_RES_NUMARGS_WID + 1))
/* arg 1 of obtain function must == arg 2 of release func */
#define EH_RES_A1_EQU_A2   (1L << (EH_RES_NUMARGS_WID + 2))
/* release function requires handle as arg 2: release(xxx, hndl) */
#define EH_RES_HNDL_2ND    (1L << (EH_RES_NUMARGS_WID + 3))
/* null resource handle may be passed to re-obtaining function */
#define EH_RES_ALLOW_NULL  (1L << (EH_RES_NUMARGS_WID + 4))
#define EH_RES_ALLOW_ZERO  EH_RES_ALLOW_NULL
#if 0
/* release(EHRESLIST *) for customization */
#define EH_RES_REL_RESLIST (1L << (EH_RES_NUMARGS_WID + 5))
/* call: release(hndl, NULL) */
#define EH_RES_REL_H_NULL  (1L << (EH_RES_NUMARGS_WID + 6))
/* call: release(hndl, arg1) */
#define EH_RES_REL_H_ARG1  (1L << (EH_RES_NUMARGS_WID + 7))
#endif

bool    ehResObt      A((const char*,const char*,int,EHRESTRK*,int,const char*,
                        pfv_ulv,ulong,...));
bool    ehResReObt    A((const char*,const char*,int,EHRESTRK*,int,const char*,
                        ulong,pfv_ulv,ulong,...));
void    ehResRel      A((const char*,const char*,int,EHRESTRK*,int,pfv_ulv,
                        ulong,...));
bool    ehResVerify   A((EHRESTRK *pEhResTrk, ulong ulHndl, int iType));
pfv_ulv ehResRelBy    A((EHRESTRK *pEhResTrk, ulong ulHndl, int iType));
void    ehResSetRelBy A((EHRESTRK *pEhResTrk, ulong ulHndl, int iType,
                        pfv_ulv pfvRelBy));
bool    ehResOnRel    A((EHRESTRK *pEhResTrk, ulong ulHndl1, int iType1,
                                              ulong ulHndl2, int iType2));

void ehBndSet A((void *pvMem, size_t szMem));

#endif /* EH_H */
