/***

ehchnvol.c -- Error Handler, support for channels and volumes

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char *rcsid =
       "$Id: ehchnvol.c,v 1.4 1996/05/12 15:47:18 ccobb Exp $";

#undef  EHSIMCTRLC
#include "ehchnvol.h"

/***
IPC_IMPLEMENTED is set to 1 if 
some type of ipc (shared memory?)
allows a remote process to control
the behavior of the error handler
in this process.
***/
/* #if ehMSDOS || ehAMIGA || ehBEOS */
#if defined(HAVE_UNISTD_H) && defined(HAVE_SYS_IPC_H) && defined(HAVE_SYS_SHM_H)
#define IPC_IMPLEMENTED 1
#else
#define IPC_IMPLEMENTED 0
#endif

EHBEGINEXTERNC

#include <stdio.h>
#if !ehMX && !ehSTMC
#include <sys/types.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_SYS_IPC_H
#include <sys/ipc.h>
#endif

#ifdef HAVE_SYS_SHM_H
#include <sys/shm.h>
#endif

EHENDEXTERNC

const char *apcEhChan[EH_NUMCHAN] =
{
   "unit", "alpha", "beta" /* unit, alpha, beta */
};

const char *apcEhVol[EH_NUMVOL] =
{ 
   "ERR", "WRN", "REM", "DBG" /* errors, warnings, remarks & debug */
};

/*** public functions ***/
#if ehSUNOS
char *shmat  A((int shmid, char *chmaddr, int shmflg));
int   shmctl A((int shmid, int cmd, struct shmid_ds *buf));
int   shmdt  A((char *shmaddr));
int   shmget A((key_t key, int size, int shmflg));
#endif

/* sole access to channel volume level */
short *phEhChnVol = NO_CHNVOL;

#if IPC_IMPLEMENTED
static int   iShmId = -1;
#endif

/***
ehInitChnVol()
Obtain shared memory for channel volume level.
***/

#if !IPC_IMPLEMENTED
int
ehInitChnVol A1(pid_t, iPID)
{
   static short hChnVol = 0;

   // puts("ehInitChnVol: entering (1)");

   phEhChnVol = &hChnVol;
   NOREF(iPID);

   // puts("ehInitChnVol: leaving (1)");

   return(0);
}

#else /* IPC_IMPLEMENTED */

int
ehInitChnVol A1(pid_t, iPID)
/* {PID|0} if 0, use current process id and IPC_CREAT */
/*         if PID, use it and DON'T use IPC_CREAT */
{
   int iErr = 0;
   key_t ktKey;
   int iMode;

   // puts("ehInitChnVol: entering (2)");

   if (iPID == EH_NEWDBGVAL)/* use current PID and create new dbgval */
   {
      iPID = getpid();
      iMode = IPC_CREAT | 0660;
   }
   else  /* use given PID, find dbgval for that PID (no IPC_CREAT) */
   {
      iMode = 0660;
   }

   ktKey = (key_t)(EH_PIDPREFIX + iPID);
   if ((iShmId = shmget(ktKey, sizeof(*phEhChnVol), iMode)) == -1)
   {
      perror("ehchnvol.c->shmget()");
      fprintf(EHSTDERR, "shmget() failed for pid %d\n", iPID);
      fflush(EHSTDERR);
      iErr = 1;
      goto Exit;
   }

   if ((phEhChnVol = (short *)shmat(iShmId, (char *)NULL, 0))
         == NO_CHNVOL)
   {
      perror("ehInitChnVol->shmat");
      iErr = 1;
      goto Exit;
   }

   if (iMode & IPC_CREAT)
   {
      *phEhChnVol = EH_VOL_ERR;
   }

Exit:
   if (iErr)
   {
      ehTermChnVol();
   }

   // puts("ehInitChnVol: leaving (2)");

   return(iErr);
}

#endif /* ehMSDOS */

/***
ehTermChnVol()
Obtain shared memory for channel volume level.
***/

#if !IPC_IMPLEMENTED

void ehTermChnVol A0()
{
}

#else /* IPC_IMPLEMENTED */

void
ehTermChnVol()
{
   if (phEhChnVol == NO_CHNVOL)
   {
      if (shmdt((char *)phEhChnVol))
      {
         fprintf(EHSTDERR, "shmdt() failed\n");
         fflush(EHSTDERR);
         goto Exit;
      }
      phEhChnVol = NO_CHNVOL;
   }
   if (iShmId != -1)
   {
      if (shmctl(iShmId, IPC_RMID, NULL))
      {
         perror("smctl() failed");
         goto Exit;
      }
      iShmId = -1;
   }
Exit:
   return;
}
#endif /* IPC_IMPLEMENTED */


#ifndef ehGetVol

/***
ehGetVol()
Get channel volume level from shared memory.
***/

int 
ehGetVol A1(int, iChan)   /* {0..EH_NUMCHAN} which channel to change */
{
   int iVol = EH_VOL_ERR; /* default to ERR volume */

   if (phEhChnVol == NO_CHNVOL)
   {
      goto Exit;
   }

   /*** get the channel priority ***/
   iVol = (int)(*phEhChnVol >> (EH_BITS_PER_CHAN * iChan)) & EH_VOL_MASK;

Exit:
   return(iVol);
}

#endif

/***
ehSetVol()
Set channel volume level in shared memory.
***/

int 
ehSetVol A2(int, iChan, 
            int, iVol)
             /* {0..15} which channel to change */
             /* {0..3,-1} new volumes for channel */
{
   int iErr = 0;

   if (phEhChnVol == NO_CHNVOL)
   {
      iErr = 1;
      goto Exit;
   }

   /*** set the channel priority to new volume ***/
   /* rm old bits */
   *phEhChnVol &= ~(EH_VOL_MASK << (EH_BITS_PER_CHAN * iChan));

   /* merge */
   *phEhChnVol |= (iVol & EH_VOL_MASK) << (EH_BITS_PER_CHAN * iChan); 

Exit:
   return(iErr);
}


#ifndef ehGetFun

/***
ehGetFun()
Test Function Tracing Bit.
***/

int 
ehGetFun A1(int, iChan)
{
   int iBit = 0;

   if (phEhChnVol == NO_CHNVOL)
   {
      goto Exit;
   }

   /*** get the channel priority ***/
   iBit = !!((*phEhChnVol >> (EH_BITS_PER_CHAN * iChan)) & EH_FUN_BIT);

Exit:
   return(iBit);
}

#endif

/***
ehSetFun()
Set channel function tracing bit in shared memory.
***/

int 
ehSetFun A2(int, iChan, 
            int, iFunBit)
             /* {0..EH_NUMCHAN} which channel to change */
             /* {0|1} new bits for channel */
{
   int iErr = 0;

   if (phEhChnVol == NO_CHNVOL)
   {
      iErr = 1;
      goto Exit;
   }

   /*** set/reset the function tracing bit ***/
   if (iFunBit)
   {
      *phEhChnVol |= EH_FUN_BIT << (EH_BITS_PER_CHAN * iChan);
   }
   else
   {
      *phEhChnVol &= ~(EH_FUN_BIT << (EH_BITS_PER_CHAN * iChan));
   }


Exit:
   return(iErr);
}

#ifndef ehGetMem

/***
ehGetMem()
Test Memory Tracing Bit.
***/

int 
ehGetMem A1(int, iChan)
{
   int iMemVal = 0;

   if (phEhChnVol == NO_CHNVOL)
   {
      goto Exit;
   }

   /*** get the memory value ***/
   iMemVal = (int)(*phEhChnVol >> (EH_BITS_PER_CHAN * iChan)) & EH_MEM_MASK;
   iMemVal >>= EH_BIT_OFFSET;

Exit:
   return(iMemVal);
}

#endif

/***
ehSetMem()
Set channel memory tracing bit in shared memory.
***/

int 
ehSetMem A2(int, iChan, 
            int, iMemVal)
             /* {0..EH_NUMCHAN} which channel to change */
             /* {0..3} new setting for memory tracing bit */
{
   int iErr = 0;

   if (phEhChnVol == NO_CHNVOL)
   {
      iErr = 1;
      goto Exit;
   }

   /*** set the new memory value ***/

   /* rm old bits */
   *phEhChnVol &= ~(EH_MEM_MASK << (EH_BITS_PER_CHAN * iChan));

   /* merge */
   iMemVal <<= EH_BIT_OFFSET;
   *phEhChnVol |= (iMemVal & EH_MEM_MASK) << (EH_BITS_PER_CHAN * iChan); 

Exit:
   return(iErr);
}

#ifndef ehGetOut

/***
ehGetOut
Test Output File bit.
***/

int 
ehGetOut A0()
{
   int iOutVal = 0; /* {1=stdout|2=stderr} */

   if (phEhChnVol == NO_CHNVOL)
   {
      goto Exit;
   }

   /*** get the output file value ***/
   iOutVal = (int)(*phEhChnVol >> (EH_BITS_PER_CHAN * EH_NUMCHAN)) & 1;

   if (iOutVal == 0) iOutVal = 2; /* stderr */

Exit:
   return(iOutVal);
}

#endif

/***
ehSetOut()
Set Output File selection bit.
***/

int 
ehSetOut A1(int, iOutVal)
             /* {1=stdout|2=stderr} new setting for output file selection bit */
{
   int iErr = 0;

   if (phEhChnVol == NO_CHNVOL)
   {
      iErr = 1;
      puts("ehSetOut: error");
      goto Exit;
   }

   /*** set the new output file bit ***/
   if (iOutVal == 2) iOutVal = 0;
   if (iOutVal)
   {
      *phEhChnVol |= 1 << (EH_BITS_PER_CHAN * EH_NUMCHAN);
   }
   else
   {
      *phEhChnVol &= ~(1 << (EH_BITS_PER_CHAN * EH_NUMCHAN));
   }

Exit:
   return(iErr);
}

