/***

ehenv.h -- Error Handler, environment auto-detection

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

#ifndef EHENV_H
#define EHENV_H 1
static const char*ehenv_h =
       "$Id: ehenv.h,v 1.10 1996/05/12 15:47:35 ccobb Exp $";

#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE 1
#endif


/***

The Error Handler uses two ways to determine environment-specific features.

For un*x-like systems which can support /bin/sh script files, the 'configure'
script may be used.  This script will examine the current environment and
determine what features are available.  Conditional compilation within
the Error Handler source files will make use of these features where
avilable.  If this maechanism was used, the preprocessor symbol HAVE_CONFIG_H
will be defined.  The corresponding environmental definitions will be found
in the file ehconfig.h.

For non-un*x-like systems or those that do not support /bin/sh scripts,
this file <ehenv.h> attempts to automatically detect the identity of the
current compilation environment.  Several preprocessor symbols are defined
to help identify the current environment.  These symbols are then tested at
appropriate points in the Error Handler source code to enable or disable
certain features.  Unfortunately, this method requires hand-tailoring this
file for new porting environments.

***/


/***
 ***
 *** HAVE_CONFIG_H
 ***
 ***/

#ifdef HAVE_CONFIG_H

#include "ehconfig.h"

/* Unix-like operating systems */
#define ehNIX     1

/***

all appropriate environmental definitions
are passed via the 'configure'-generated
Makefile to the compiler command line.

Obsolete definitions for un*x-like systems.

|*** compilers ***|

|* AIX "XL" C *|
#if defined(_AIX) && defined(_IBMR2)
static char cc[] = "cc: AIX XL C";
#define ehXLC    1
#define ehXLCVER 140
#else
#define ehXLC 0
#endif

|* Sun K&R C *|
#if defined(sun) && !defined(__STDC__) && !defined(__GNUC__)
static char cc[] = "cc: SUN K&R C";
#define ehSUNC 1
#define ehSUNCVER   410
#define const
#define volatile
#else
#define ehSUNC 0
#endif

|* Solaris (SparcWorks) C *|
#if defined(sun) && defined(sparc) && defined(__STDC__) && !defined(__cplusplus) && !defined(__GNUC__)
static char cc[] = "cc: SOLARIS C";
#define ehSOLC 1
#define ehSOLCVER   500
#else
#define ehSOLC 0
#endif

|* Solaris (SparcWorks) C++ *|
#if defined(sun) && defined(spark) && defined(__cplusplus)
static char cc[] = "cc: SOLARIS C++";
#define ehSOLCPP 1
#define ehSOLCVER   500
#else
#define ehSOLCPP 0
#endif

|*** operating system ***|

|* AIX *|
#if ehXLC
static char os[] = "os: AIX";
#define ehAIX    1
#else
#define ehAIX    0
#endif

|* BeOS *|
#if defined(ehMWCC) && !defined(ehMACOS)
static char os[] = "os: BeOS";
#define ehBEOS  1     |* BeOS *|
#else
#define ehBEOS  0     |* BeOS *|
#endif

#if defined(M_UNIX)
static char os[] = "os: SCO UNIX System V Release 3";
#define ehSCUNIX 1     |* SCO UNIX *|
#else
#define ehSCUNIX 0     |* SCO UNIX *|
#endif

#if defined(sun) && defined(sparc) && defined(unix) && defined(__svr4__)
|* && (ehSOLC || ehSOLCPP) *|
static char os[] = "os: SOLARIS";
#define ehSOLAR  1     |* SUN OS -- SOLARIS *|
#else
#define ehSOLAR  0     |* SUN OS -- SOLARIS *|
#endif

#if (defined(sun) || defined(__sun__)) && (ehSUNC || ehGNUC) && !ehSOLAR
static char os[] = "os: Sun OS";
#define ehSUNOS  1     |* SUN OS -- 4.x *|
#else
#define ehSUNOS  0     |* SUN OS -- 4.x *|
#endif

***/

/***
 ***
 ***/
#else /* HAVE_CONFIG_H not defined -- used auto-detect mechanism */
/***
 ***
 ***/



/***
 ***
 *** Auto-detect a compiler
 ***
 ***/

/* Borland C */
#if defined(__BORLANDC__)
static char cc[] = "cc: Borland C++";
#define ehBC 1
#define ehBCVER   452  /* {300|310|452} */
#ifdef __cplusplus
#define ehEXCEPTIONHANDLING 1
#endif
#else
#define ehBC 0
#endif

/* Amiga DICE C */
#if defined(_DCC)
static char cc[] = "cc: DICE C";
#define ehDCC 1
#define ehDCCVER   206
#else
#define ehDCC 0
#endif

/* GNU C */
#if defined(__GNUC__)
static char cc[] = "cc: GNU C";
#define ehGNUC    1
#define ehGNUCVER 272
#ifndef __STDC__
#define const
#define volatile
#endif
#else
#define ehGNUC 0
#endif

/* Manx Aztec C */
#if defined(AZTEC_C)
static char cc[] = "cc: Manx C";
#define ehMX 1
#define ehMXVER   600
#else
#define ehMX 0
#endif

/* Microsoft C */
#if defined(M_I86)
static char#] = "cc: Microsoft C";
#define ehMSC 1
#define ehMSCVER   600
#else
#define ehMSC 0
#endif

/* SAS C */
#if defined(__SASC_60) && !defined(__SASCXX)
static char cc[] = "cc: SAS C";
#define ehSASC 1
#define ehSASCVER   600
#else
#define ehSASC 0
#endif

/* SAS C++: this is a kludge; __SASCXX is defined on the command line */
#if defined(__SASCXX)
static char cc[] = "cc: SAS C++";
#define ehSCXX 1
#define ehSCXXVER   650
#ifdef __FUNC__ /* SASC C++ defines __FUNC__ as "" for some reason */
#undef __FUNC__
#endif
#else
#define ehSCXX 0
#endif

/* StormC++ */
#if defined(__STORM__)
static char cc[] = "cc: Stormc++";
#define ehSTMC 1
#define ehSTMCVER   105
#else
#define ehSTMC 0
#endif

/* Metrowerks CC */
#if defined(__MWERKS__)
static char cc[] = "cc: Metrowerks CC";
#define ehMWCC 1
#define ehMWCCVER   000
#ifdef __cplusplus
#define ehEXCEPTIONHANDLING 1
#endif
#else
#define ehMWCC 0
#endif

#if ((ehXLC+ehBC+ehDCC+ehGNUC+ehMX+ehMSC+ehSASC+ehSCXX+ehSUNC+ehSOLC+ehSOLCPP+ehSTMC+ehMWCC) == 0)
one compiler must be selected
#endif


/***
 ***
 *** Auto-select the target operating system
 ***
 ***/

/* Unix-like operating systems */
#define ehNIX     0

/* MAC OS */
#if !defined(ehBEOS) && ( defined(macintosh) || defined(THINK_C) )
static char os[] = "os: Mac Finder";
#define ehMACOS  1     /* MAC OS */
#else
#define ehMACOS  0     /* MAC OS */
#endif

/* DOS */
#if defined(MSDOS) || defined(__MSDOS__)
static char os[] = "os: MS/PC/DR-DOS";
#define ehMSDOS  1     /* IBM-PC BIOS COMPATIBLE */
#else
#define ehMSDOS  0     /* IBM-PC BIOS COMPATIBLE */
#endif

/* this entry must go after ehSUNOS */
#if defined(_AMIGA)||defined(AMIGA)||(ehGNUC&&!ehSUNOS&&!ehSOLAR)||\
 defined(MCH_AMIGA)||ehSCXX||ehSTMC
static char os[] = "os: Amiga OS";
#define ehAMIGA  1     /* Amiga Operating System */
#else
#define ehAMIGA  0     /* Amiga Operating System */
#endif

/* BeOS */
#if ehMWCC && !ehMACOS
static char os[] = "os: BeOS";
#define ehBEOS  1     /* BeOS */
#else
#define ehBEOS  0     /* BeOS */
#endif

#if ((ehAIX+ehAMIGA+ehMACOS+ehPCDOS+ehSCUNIX+ehSOLAR+ehSUNOS+ehBEOS) == 0)
one operating_system must be selected
#endif

#endif /* CONFIGURE_WAS_ not _USED */

/***
 ***
 *** Auto-select a processor
 ***
 ***/

/*** Intel ***/

/* Intel 80386 */
#if defined(i386) || ehPCDOS
static char cpu[] = "cpu: 386";
#define ehI386   1
#else
#define ehI386   0
#endif

/* Intel 80486 */
#if defined(i486)
static char cpu[] = "cpu: 486";
#define ehI486   1
#else
#define ehI486   0
#endif

/* Intel Family Characteristics */
#if ehI386||ehI486
#define ehINTEL  1
#define ehNOALIGN  1   /* no alignment required */
#define ALIGNTYP char
#endif

/*** Motorola ***/

/* Motorola 68040 */
#if defined(mc68040) || defined(__mc68040__)
static char cpu[] = "cpu: 68040";
#define ehM68040   1
#else
#define ehM68040   0
#endif

/* Motorola 68030 */
#if defined(mc68030) || defined(_M68030) || defined(__mc68030__)
static char cpu[] = "cpu: 68030";
#define ehM68030   1
#else
#define ehM68030   0
#endif

/* Motorola 68020 */
#if (ehM68040||ehM68030) == 0
#if defined(mc68020) || defined(_M68020) || defined(__mc68020__)
static char cpu[] = "cpu: 68020";
#define ehM68020   1
#else
#define ehM68020   0
#endif
#endif

/* Motorola 68000 */
#if (ehM68040||ehM68030||ehM68020) == 0
#if ( defined(mc68000) || defined(_M68000) || defined(MPU68000) || \
 defined(__mc68000__) || defined(ehSTMC) ) && !defined(ehBEOS)
static char cpu[] = "cpu: 68000";
#define ehM68000   1
#else
#define ehM68000   0
#endif
#endif

/* Motorola Family Characteristics */
#if  ehM68000||ehM68030||ehM68040
#define ehMOTOROLA   1
#define ehNOALIGN    1   /* no alignment required for 68020 */
#define ALIGNTYP   char
#endif

/*** IBM RISC/POWERPC ***/

/* IBM RISC 6000 */
#if defined(_IBMR2)
static char cpu[] = "cpu: RS6000";
#define ehRS6000 1   /* IBM RISC/SYSTEM (RS) 6000 */
#else
#define ehRS6000 0   /* IBM RISC/SYSTEM (RS) 6000 */
#endif

/* Other POWER PC */
#if defined(__POWERPC__)
static char cpu[] = "cpu: POWERPC";
#define ehPPC 1   /* MAC/BeBox */
#else
#define ehPPC 0   /* MAC/BeBox */
#endif

/* IBM POWER-PC Family Characteristics */
#if ehRS6000 || ehPPC
#define ehPOWERPC   1
#define ehNOALIGN   1   /* no alignment required */
#define ALIGNTYP char
#endif

/*** SPARC ***/

/* TI SPARC 2 */
#if defined (sparc) || defined (__sparc__)
static char cpu[] = "cpu: sparc2";
#define ehSparc2   1   /* Sun Sparc Station 2 */
#else
#define ehSparc2   0
#endif

/* TI SPARC 10 */
#if defined(sparc10)
static char cpu[] = "cpu: sparc10";
#define ehSparc10  1   /* Sun Sparc Station 10 */
#else
#define ehSparc10  0
#endif

/* Sparc Family Characteristics */
#if ehSparc2||ehSparc10
#define ehSPARC 1
#define ehNOALIGN 1   /* no alignment required */
#define ALIGNTYP char
#endif

#if ((ehI386+ehI486+ehM68000+ehM68030+ehM68040+ehRS6000+ehSparc2+ehSparc10+ehPPC) == 0)
one processor must be selected
#endif

/* short cut for extern "C" */
#ifdef __cplusplus
#define EHBEGINEXTERNC extern "C" {
#define EHENDEXTERNC }
#else
#define EHBEGINEXTERNC
#define EHENDEXTERNC
#endif

#endif /* EH_H */

