/***

ehercode.c -- Error Handler, ER code conversions

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char *rcsid =
       "$Id: ehercode.c,v 1.2 1995/04/09 08:30:42 ccobb Exp $";

#include "ehercode.h"
#include "ehstr2lo.h"

EHBEGINEXTERNC
#include <string.h>
#include <ctype.h>
EHENDEXTERNC

EH      Eh = { "unknown" };    /* global variables */

ulong       /* hashed error value as it appears in xxerrmsg.h */
ehHash A1(  /* "hash" an ER struct to a ulong */
   ER, Er)  /* contains the prefix and error code */
{
   ulong ulHash = 0;
   int iLen = strlen(Er.acPrefix);
   bool error = false;
   char c;

   //fprintf(stdout, "iLen = %d\n", iLen);
   if (iLen < 1 || iLen > 4)
   {
      if (iLen != 0) /* zero length may indicate a null error, so its ok */
         fprintf(stdout,
            "Prefix string must be from 1 to 4 characters long: \"%s\"\n",
            Er.acPrefix);
      return ~0L;
   }
   switch (iLen)
   {
   case 4:
      if (!( isalpha(Er.acPrefix[3]) ||
             isdigit(Er.acPrefix[3]) ))
         error = true;
      /*FALLTHRU*/
   case 3:
      if (!( isalpha(Er.acPrefix[2]) ||
             isdigit(Er.acPrefix[2]) ))
         error = true;
      /*FALLTHRU*/
   case 2:
      if (!( isalpha(Er.acPrefix[1]) ||
             isdigit(Er.acPrefix[1]) ))
         error = true;
      /*FALLTHRU*/
   case 1:
      if (!( isalpha(Er.acPrefix[0]) ))
         error = true;
   }
   if (error)
   {
      fprintf(stdout, "Prefix string \"%s\" has invalid format\n",
            Er.acPrefix);
      fprintf(stdout, "The prefix must have the following format:\n");
      fprintf(stdout, "\t1st char - letter (a-z) only\n");
      fprintf(stdout, "\t2nd char - letter or number (0-9)\n");
      fprintf(stdout, "\t3rd char - letter or number\n");
      fprintf(stdout, "\t4th char - letter or number\n");
      return ~0L;
   }

   if (Er.hCode < 0) Er.hCode = -Er.hCode;
   if (Er.hCode > 3260)
   {
      fprintf(stdout,
            "Code %d must be less than 3260\n", Er.hCode);
      return ~0L;
   }

   /*** calculate hash code ***/

   ehStr2Lo(Er.acPrefix);

   /* [0] */
   ulHash = (Er.acPrefix[0] - 'a');  /* 0 - 25 */

   /* [1] */
   if (iLen > 1)
   {
      if (isalpha(c = Er.acPrefix[1]))
         ulHash += ((c - '`')      * 26L);
      else
         ulHash += ((27 + c - '0') * 26L);

      if (iLen > 2)
      {
         /* [2] */
         if (isalpha(c = Er.acPrefix[2]))
            ulHash += ((c - '`')      * 962L);
         else
            ulHash += ((27 + c - '0') * 962L);

         if (iLen > 3)
         {
            /* [3] */
            if (isalpha(c = Er.acPrefix[3]))
               ulHash += ((c - '`')      * 35594L);
            else
               ulHash += ((27 + c - '0') * 35594L);
         }
      }
   }

   ulHash += (Er.hCode * 1316978L);

   return (ulHash);
}


ER                /* return unhashed ER struct */
ehUnHash A1(      /* unhash a ulong error (hash) code into an ER struct */
   ulong, ulHash) /* ulong error (hash) code */
{
   ER    Er;
   ulong ulDividend;

   Er.hCode = (short)(ulHash / 1316978L);
   if (Er.hCode == 1) Er.hCode = -1; /* special case for Ctrl C */
   ulHash %= 1316978L;

   ulDividend = ulHash / 35594L;
   if (ulDividend == 0)
      Er.acPrefix[3] = (char)0;
   else if (ulDividend < 27)
      Er.acPrefix[3] = (char)('`' + ulDividend);
   else 
      Er.acPrefix[3] = (char)('0' + ulDividend - 27);
   ulHash %= 35594L;

   ulDividend = ulHash / 962L;
   if (ulDividend == 0)
      Er.acPrefix[2] = (char)0;
   else if (ulDividend < 27)
      Er.acPrefix[2] = (char)('`' + ulDividend);
   else
      Er.acPrefix[2] = (char)('0' + ulDividend - 27);
   ulHash %= 962L;

   ulDividend = ulHash / 26L;
   if (ulDividend == 0)
      Er.acPrefix[1] = (char)0;
   else if (ulDividend < 27)
      Er.acPrefix[1] = (char)('`' + ulDividend);
   else
      Er.acPrefix[1] = (char)('0' + ulDividend - 27);
   ulHash %= 26L;

   Er.acPrefix[0] = (char)('a' + ulHash);

   return (Er);
}


bool                /* current error matches proposed error */
ehErrorIs A2(       /* unhash a ulong error (hash) code into an ER struct */
   ulong, ulOffset, /* ulong error (hash) code */
   ulong, ulHash)   /* ulong error (hash) code */
{
   NOREF(ulOffset); /* not referenced */
   return(ehHash(Eh.Er) == ulHash);
}

void
dmpER A1(
   ER, Er)
{
   puts("----- begin ER -----");
   printf("acPrefix = \"%.4s\"\n", Er.acPrefix);
   printf("hCode    = %d\n",       Er.hCode);
   puts("-----  end ER  -----");
}

