/***

ehextini.c - Error Handler, external initialization vector

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char *rcsid =
       "$Id: ehextinehextini.c,v 1.3 1996/04/26 19:56:23 ccobb Exp $";

#include "ehextini.h"

/***
   This function pointer points to a function which
   performs external environment-specific initialization.
   When building standard C applications using eh,
   eh performs all initialization internally and so
   the value of this pointer can be set to NULL.  If
   building an application which makes use of
   non-"Standard C" features which are specific to a
   particular environment, such as external resource
   tracking, some environment-specific initialization
   may be needed.  In this case, the funtion pointer
   should be initialized to point to a function which
   performs the needed environment-specific
   initialization.  The method of doing this which
   is least imposing on the application code is to 
   have a separate link library to support that external
   environment.  That library will contain an identical
   copy of this pointer which will pre-empt this copy
   when that library is linked.  The external library
   should always appear before this normal eh library
   in the link list.

   The copy of this pointer should be statically
   initiallized in that library as it is here, but to a
   non-null value.  When that library is linked into an
   application, it will automatically override the default null
   value that is defined here, and the desired initialization
   function will be called.  See ehosinit.c.
***/

void (*pfvEhExtInit)A0() = NULL; /* external initialization vector */
