/***

ehintini.c -- Error Handler, internal initialization

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char *rcsid =
       "$Id: ehintini.c,v 1.13 1996/05/12 15:48:18 ccobb Exp $";

#include "ehintini.h"

#if ehAMIGA
#include "ehamiga.h"
#endif /* ehAMIGA */

#include "eherrs.h"
#include "ehintres.h"

/***
   Slash used in path names.
***/
const char *pcEhSlash =
#if ehMSDOS
   "\\";
#else
   "/";
#endif

/***

This function contains any internal (os-specific)
initialization that takes place in EHMAIN()
after ehPgmEnt() and ehFunEnt() are called.
This is therefore written as a full/normal eh-compliant
function.

This function is called via the function pointer
pfvEhIntInit to replicate the same structure that
is needed for external initialization.

***/

void
ehIntInit A0()
{
   // puts("ehIntInit: about to call EHTRY(ehIntInit)");

   EHTRY(ehIntInit) {

   // puts("ehIntInit: back from EHTRY(ehIntInit)");

/***
   the following conditional compilation(s)
   represent code needed to properly set up
   the STANARD C environment.  This may or may
   not be the same as code needed to set up a
   non-standard C environment.  Access to 
   environment-specific features is sometimes
   needed to properly setup operation of the
   Error Handler.
***/

#if ehAMIGA

      /* first set up the recovery routine -- in case of a failure */
      ehAtExit(ehAmResetExceptCode);  /* at term time */

      /* now set up control-c handling code */
      ehAmSetExceptCode(ehAmSigInt, (ULONG)SIGBREAKF_CTRL_C, (void*)NULL);

#endif

      /* first set up the termination routine -- in case of a failure */
      // puts("ehIntInit: about to call ehAtExit(ehIntResTerm)");
      ehAtExit(ehIntResTerm);  /* at term time */

      /* now initialize internal resource tracking */
      // puts("ehIntInit: about to call ehIntResInit()");
      ehIntResInit();

   } EHCATCH;

   EHLEAVE();
} /* ehIntInit */

