/***

ehintres.c -- Error Handler, internal resource tracking

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char*rcsid =
       "$Id: ehintres.c,v 1.3 1996/01/08 17:30:26 ccobb Exp $";

#define EH_NO_MEMORY_TRACKING 1
#include "ehintres.h"

EHFILENAME("ehintres.c");

EHRESTRK ehResTrk; /* internal resource tracking anchor */

/***
ehIntResInit()
Initialize internal resource tracking data structures.
***/

void
ehIntResInit(void)
{
   volatile bool gotStdin  = false;
   volatile bool gotStdout = false;

   EHTRY(ehIntResInit) {

      // puts("ehIntResInit: about to call ehResInit()");
      ehResInit(&ehResTrk);

      /* now mark as open stdin, stdout and stderr */

      gotStdin = ehResObt(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)EH_RES_FILESTR, "stdin",
            (pfv_ulv)0, 1L,
            (ulong)stdin);

      gotStdout = ehResObt(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)EH_RES_FILESTR, "stdout",
            (pfv_ulv)0, 1L,
            (ulong)stdout);

      ehResObt(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)EH_RES_FILESTR, "stderr",
            (pfv_ulv)0, 1L,
            (ulong)stderr);                                             

      // puts("ehIntResInit: about to call EHLEAVE()");

   } EHCATCH;

   if (ehError())
   {
      if (gotStdout)
      {
         ehResRel(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
               (int)EH_RES_FILESTR, (pfv_ulv)0, 1L,
               (ulong)stdout);
      }
      if (gotStdin)
      {
         ehResRel(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
               (int)EH_RES_FILESTR, (pfv_ulv)0, 1L,
               (ulong)stdin);
      }
   }

   EHLEAVE();
} /* ehIntResInit */


/***
ehIntResTerm()
Terminate internal resource tracking.
***/

#include "ehresini.h"
#if     TRACKING_INTERNAL_MEMORY_USE
#include <mt.h>
#endif

void
ehIntResTerm(void)
{
   EHTRY(ehIntResTerm) {} EHCATCH;

   if (ehResVerify(&ehResTrk, (ulong)stderr, (int)EH_RES_FILESTR))
   {
      ehResRel(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)EH_RES_FILESTR, (pfv_ulv)0, 1L,
            (ulong)stderr);
   }
   if (ehResVerify(&ehResTrk, (ulong)stdout, (int)EH_RES_FILESTR))
   {
      ehResRel(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)EH_RES_FILESTR, (pfv_ulv)0, 1L,
            (ulong)stdout);
   }
   if (ehResVerify(&ehResTrk, (ulong)stdin, (int)EH_RES_FILESTR))
   {
      ehResRel(__FILE__,_EHFUNC_,__LINE__,&ehResTrk,
            (int)EH_RES_FILESTR, (pfv_ulv)0, 1L,
            (ulong)stdin);
   }
 
   ehResTerm(&ehResTrk);
#if     TRACKING_INTERNAL_MEMORY_USE
   mtChkMem();
#endif
   EHLEAVE();
} /* ehIntResTerm */


#include "ehintini.h"

/***
   This function pointer points to the function which
   performs internal resource initialization.  It is not
   really necessary.  The internal initialization
   function (ehIntInit()) could have been called
   directly.

   It is called through this vector so that the same
   architecture is used for internal resource initialization
   as for external resource initialization, making it easier to
   understand (or duplicate) the resource management process.
***/

void (*pfvEhIntInit)A0() = ehIntInit; /* internal initialization vector */
