/***

ehlist.c -- Error Handler, linked list manager

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char*rcsid =
       "$Id: ehlist.c,v 1.5 1996/03/04 06:51:14 ccobb Exp $";

#include "ehlist.h"
#include "eherrs.h"

/* unnamed/non-prioritized list support */

bool
ehAddHead A2(EHLIST *, pEhList,
             EHNODE *, pEhNode)
{
   EHNODE *pEhNodeHead = pEhList->pEhNodeHead;  /* current head of the list */

   //puts("ehAddHead: about to invoke EHTRACE(ehAddHead)");
   EHTRACE(ehAddHead);

   //puts("ehAddHead: about to call EHASSERT_LEAVE(pEhList != NULL)");
   EHASSERT_RETURN(pEhList != NULL, false);

   //puts("ehAddHead: about to call EHASSERT_LEAVE(pEhNode != NULL)");
   EHASSERT_RETURN(pEhNode != NULL, false);

   //printf("ehAddHead: adding node 0x%p to list 0x%p\n", pEhList, pEhNode);

   /* take care of new node */
   pEhNode->pEhNodeSucc = pEhNodeHead;
   pEhNode->pEhNodePred = (EHNODE *)pEhList;

   /* take care of old head of list */
   pEhNodeHead->pEhNodePred = pEhNode;

   /* take care of list header */
   pEhList->pEhNodeHead = pEhNode;

   //ehCallStack();
   //puts("ehAddHead: about to invoke EHLEAVE()");
   EHRETURN(true);
} /* ehAddHead */


bool
ehAddTail A2(EHLIST *, pEhList,
             EHNODE *, pEhNode)
{
   EHNODE *pEhNodeTail = pEhList->pEhNodeTailPred;  /* this is the current tail of the list */

   EHTRACE(ehAddTail);

   EHASSERT_RETURN(pEhList != NULL, false);
   EHASSERT_RETURN(pEhNode != NULL, false);

   /* take care of new node */
   pEhNode->pEhNodeSucc = (EHNODE *)&(pEhList->pEhNodeTail);
   pEhNode->pEhNodePred = pEhNodeTail;

   /* take care of old tail of list */
   pEhNodeTail->pEhNodeSucc = pEhNode;

   /* take care of list header */
   pEhList->pEhNodeTailPred = pEhNode;

   EHRETURN(true); /* function succeeded */
} /* ehAddTail */


void
ehInsert A3(EHLIST *, pEhList,
            EHNODE *, pEhNode,
            EHNODE *, pEhNodePred)
{
   EHNODE *pEhNodeSucc = pEhNodePred->pEhNodeSucc;

   EHTRACE(ehInsert);

   EHASSERT_LEAVE(pEhList != NULL);
   EHASSERT_LEAVE(pEhNode != NULL);
   EHASSERT_LEAVE(pEhNodePred != NULL);

   /* take care of new node */
   pEhNode->pEhNodeSucc = pEhNodeSucc;
   pEhNode->pEhNodePred = pEhNodePred;

   /* take care of predecessor */
   pEhNodePred->pEhNodeSucc = pEhNode;

   /* take care of successor */
   pEhNodeSucc->pEhNodePred = pEhNode;

   EHLEAVE();
} /* ehInsert */


bool
ehListIsEmpty A1(EHLIST *, pEhList)
{
   EHTRACE(ehListIsEmpty);

   EHASSERT_RETURN(pEhList != NULL, false);

   if (pEhList->pEhNodeTailPred == (EHNODE *)pEhList)
   {
      EHASSERT_RETURN(pEhList->pEhNodeHead->pEhNodeSucc == NULL, false);
      EHRETURN(true);
   }

   EHRETURN(false);
} /* ehListIsEmpty */


void
ehNewList A1(EHLIST *, pEhList)
{
   EHTRACE(ehNewList);

   EHASSERT_LEAVE(pEhList != NULL);

   pEhList->pEhNodeHead     = (EHNODE *)&(pEhList->pEhNodeTail);
   pEhList->pEhNodeTail     = 0;
   pEhList->pEhNodeTailPred = (EHNODE *)pEhList;

   EHLEAVE();
} /* ehNewList */


EHNODE *
ehRemHead A1(EHLIST *, pEhList)
{
   EHNODE *pEhNodeRem;
   EHNODE *pEhNodeRemSucc;

   EHTRACE(ehRemHead);

   EHASSERT_RETURN(pEhList != NULL, (EHNODE*)NULL);
   if (ehListIsEmpty(pEhList))
   {
      //EHDBG("list is empty");
      EHRETURN((EHNODE*)NULL);
   }

   pEhNodeRem = pEhList->pEhNodeHead;
   if (pEhNodeRem == NULL) /* uninitialized list ? */
   {
      EHERR("uninitialized list");
      EHRETURN((EHNODE*)NULL); /* silent return */
   }

   pEhNodeRemSucc = pEhNodeRem->pEhNodeSucc;

   /* take care of the removed node's successor */
   pEhNodeRemSucc->pEhNodePred = (EHNODE *)pEhList;

   /* take care of the list header */
   pEhList->pEhNodeHead = pEhNodeRemSucc;

   /* take care of the removed node */
   pEhNodeRem->pEhNodeSucc = NULL;
   pEhNodeRem->pEhNodePred = NULL;

   EHRETURN(pEhNodeRem);
} /* ehRemHead */

/***
This function is a new addition and is not
available directly in the Amig Exec Library.
It Inquires about the presence of a message
without removing it from the list.
***/
EHNODE *
ehInqHead A1(EHLIST *, pEhList)
{
   EHNODE *pEhNodeInq = pEhList->pEhNodeHead;

   EHTRACE(ehInqHead);

   EHASSERT_RETURN(pEhList != NULL, (EHNODE*)NULL);

   if (ehListIsEmpty(pEhList))
      EHRETURN((EHNODE*)NULL);

   EHRETURN(pEhNodeInq);
} /* ehInqHead */


bool
ehRemove A1(EHNODE *, pEhNode)
{
   EHNODE *pEhNodePred;
   EHNODE *pEhNodeSucc;

   EHTRACE(ehRemove);

   //puts("about to call EHASSERT_LEAVE()");
   EHASSERT_RETURN(pEhNode != NULL, false);

   //puts("setting pEhNodePred");
   pEhNodePred = pEhNode->pEhNodePred;
   //puts("setting pEhNodeSucc");
   pEhNodeSucc = pEhNode->pEhNodeSucc;

   /* take care of predecessor */
   EHASSERT_RETURN(pEhNodePred, false);
   EHASSERT_RETURN(pEhNodePred->pEhNodeSucc, false);
   pEhNodePred->pEhNodeSucc = pEhNode->pEhNodeSucc;

   /* take care of successor */
   //puts("setting pEhNodeSucc->pEhNodePred");
   pEhNodeSucc->pEhNodePred = pEhNode->pEhNodePred;

   /* take care of removed node */
   //puts("setting pEhNode->pEhNodeSucc");
   pEhNode->pEhNodeSucc = NULL;
   //puts("setting pEhNode->pEhNodePred");
   pEhNode->pEhNodePred = NULL;

   //puts("about to call EHLEAVE()");
   EHRETURN(true);
} /* ehRemove */


EHNODE *
ehRemTail A1(EHLIST *, pEhList)
{
   EHNODE *pEhNodeRem = pEhList->pEhNodeTailPred;

   EHTRACE(ehRemTail);

   EHASSERT_RETURN(pEhList != NULL, (EHNODE*)NULL);
   EHASSERT_RETURN(!ehListIsEmpty(pEhList), (EHNODE*)NULL);

   /* take care of tail's predecessor */
   pEhNodeRem->pEhNodePred->pEhNodeSucc = (EHNODE *)&(pEhList->pEhNodeTail);

   /* take care of list header */
   pEhList->pEhNodeTailPred = pEhNodeRem->pEhNodePred;

   /* take care of removed node */
   pEhNodeRem->pEhNodeSucc = NULL;
   pEhNodeRem->pEhNodePred = NULL;

   EHRETURN(pEhNodeRem);
} /* ehRemTail */



/* named/prioritized list support */


void    
ehEnqueue A2(EHLIST *, pEhList,
             EHNODE *, pEhNode)
{
   EHTRACE(ehEnqueue);
   NOREF(pEhList);
   NOREF(pEhNode);
   EHLEAVE();
} /* ehEnqueue */


EHNODE *
ehFindName A2(EHLIST *, pEhList,
              byte *,   pbName)
{
   EHTRACE(ehFindName);
   NOREF(pEhList);
   NOREF(pbName);
   EHRETURN((EHNODE*)NULL);
} /* ehFindName */


/*** additional support functions not found in Amiga Exec ***/

/* locate tail */
EHNODE *
ehLocTail A1(EHLIST *, pEhList)
{
   EHTRACE(ehLocTail);

   EHASSERT_RETURN(pEhList != NULL, (EHNODE*)NULL);
   EHASSERT_RETURN(!ehListIsEmpty(pEhList), (EHNODE*)NULL);

   EHRETURN(pEhList->pEhNodeTailPred);
} /* ehLocTail */

