/***

ehmemtrk.h -- Error Handler, memory tracking (public interface)

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

#ifndef EHMEMTRK_H
#define EHMEMTRK_H 1
static const char*ehmemtrk_h =
       "$Id: ehmemtrk.h,v 1.2 1996/04/26 19:58:16 ccobb Exp $";

#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE 1
#endif

#ifndef   EH_H
#include "eh.h"
#endif

#define EH_SZBOUNDBLK ((size_t)16)

/*** memory tracking support ***/

void *_ehMalloc A((const char *pcFil, const char *pcFun, int iLin,
                  size_t szBlock));

void *_ehCalloc  A((const char *pcFil, const char *pcFun, int iLin,
                  size_t szNum, size_t szByt));

void *_ehRealloc A((const char *pcFil, const char *pcFun, int iLin,
                  void *pv, size_t sz));

void *_ehDynArr A((const char *pcFil, const char *pcFun, int iLin,
                  void **ppcArray,size_t *pszNumEl,const size_t szEl));

void  _ehFree    A((const char *pcFil, const char *pcFun, int iLin,
                  void *pvPtr));

#ifdef ehEXCEPTIONHANDLING

void *operator new   (size_t szBlk,
                  const char*pcFil, const char*pcFun, int iLin);
#ifdef HAVE_ARRAYMEM
void *operator new[] (size_t szBlk,
                  const char*pcFil, const char*pcFun, int iLin);
#endif

class EHDELETE
{
friend void operator delete   (void*);
#ifdef HAVE_ARRAYMEM
friend void operator delete[] (void*);
#endif
public:
   EHDELETE(int iChan,const char*pcFile,const char*pcFunc,int iLine);
private:
   static int         de_iChan;
   static const char *de_pcFile;
   static const char *de_pcFunc;
   static int         de_iLine;
};

#endif /* ehEXCEPTIONHANDLING */

#ifndef EH_NO_MEMORY_TRACKING

#define malloc(sz)\
 _ehMalloc(__FILE__, _EHFUNC_, __LINE__, sz)

#define calloc(num, sz)\
 _ehCalloc(__FILE__, _EHFUNC_, __LINE__, num, sz)

#define realloc(p, sz)\
 _ehRealloc(__FILE__, _EHFUNC_, __LINE__, p, sz)

#define ehDynArr(pA, piSzA, iSzE)\
 _ehDynArr(__FILE__, _EHFUNC_, __LINE__, pA, piSzA, iSzE)

#define free(p)\
 _ehFree(__FILE__, _EHFUNC_, __LINE__, p)

#ifdef ehEXCEPTIONHANDLING

#define new \
 new(__FILE__,_EHFUNC_,__LINE__)

#define delete\
 EHDELETE _ehDelete(__CHAN__,__FILE__,_EHFUNC_,__LINE__);delete

#endif /* ehEXCEPTIONHANDLING */

#else /* memory tracking NOT active */

void *ehDynArr A((void **ppv, size_t *pszNumEl, const size_t szEl));

#endif /* EH_NO_MEMORY_TRACKING */

#endif /* EHMEMTRK_H */
