/***

ehosterm.c -- Error Handler, OS termination routines

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char *rcsid =
       "$Id: ehosterm.c,v 1.13 1996/05/12 15:48:18 ccobb Exp $";

#include "ehosterm.h"
#include "eherrs.h"

#define TERMLIST struct _term_list_ /* termination function list */
TERMLIST
{
   pfv_v     pfvTermFunc;  /* termination function */
   TERMLIST *pTermList;    /* next function to call */
};

static TERMLIST *pTermListHeader; /* anchor for TERMLIST linked list */

/***

The function loads up function pointers
which get called at ehOsTerm()...
sort of like atexit().

***/

void
ehAtExit A1(pfv_v, pfvTermFunc)
{
   TERMLIST *pTermList;

   // puts("ehAtExit: about to call EHTRACE(ehAtExit)");

   EHTRACE(ehAtExit);

   // puts("ehAtExit: back from  EHTRACE(ehAtExit)");

   pTermList = NEW(TERMLIST);
   if (pTermList == NULL) EHTHROW_LEAVE(EH_NEWMACRO);

   pTermList->pfvTermFunc = pfvTermFunc; /* function to call at ehOsTerm() time */
   pTermList->pTermList = pTermListHeader; /* attach to list */
   pTermListHeader = pTermList;

   // puts("ehAtExit: about to call EHLEAVE()");

   EHLEAVE();

   // puts("ehAtExit: back from  EHLEAVE()");

} /* ehAtExit */


/***

This function executes functions that were
specified with the ehAtExit() call:
last first

***/

void
ehOsTerm A0()
{
   TERMLIST *pTermList;

   EHTRACE(ehOsTerm);

   pTermList = pTermListHeader; /* last one first */
   while (pTermList)
   {
      TERMLIST *pTL = pTermList; /* temp copy */
      (*(pTermList->pfvTermFunc))();
      pTermList = pTermList->pTermList;
      EHFREE(pTL);
   }

#if TRACKING_INTERNAL_MEMORY_USE
   mtChkMem();
#endif

   EHLEAVE();
} /* ehOsTerm */

