/***

ehsigint.c -- Error Handler, interrupt (Control-C) signal catcher

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char *rcsid =
       "$Id: ehsigint.c,v 1.6 1996/05/12 15:48:52 ccobb Exp $";

#undef  EHSIMCTRLC
#include "ehsigint.h"

EHBEGINEXTERNC
#include <signal.h>
EHENDEXTERNC

EHFILENAME("ehsigint.c");

/***
ehSigInt()
Control-C support.
***/

void
EHCALLBACK
ehSigInt A1(int, iSig)
{
   // puts("ehSigInt: entering");
   NOREF(iSig);
   
   Eh.yIntSig = 1;
   Eh.ulCtrlCIni = 0; /* no more simulated control-c's */
   ehSigInstall();    /* reinstall SIGINT handler */

   // puts("ehSigInt: leaving");
}


static void (*pfvOldSigFunc) A1(int, sig) = SIG_DFL;

/***
Install SIGINT signal handler.
***/

void
ehSigInstall A0()
{
#if !ehMX || EH_CHAN
   if (pfvOldSigFunc == SIG_DFL) /* get the actual function */
   {
      pfvOldSigFunc = signal(SIGINT, ehSigInt);
   }
   else
   {
      signal(SIGINT, ehSigInt);
   }
#endif
}

/***
Uninstall SIGINT signal handler.
***/

void
ehSigUninstall A0()
{
#if !ehMX || EH_CHAN
   signal(SIGINT, pfvOldSigFunc);
#endif
}

/***

$Log: ehsigint.c,v $
 * Revision 1.6  1996/05/12  15:48:52  ccobb
 * Added Storm C support and recompiled with dcc and sasc after finishing semester project.
 *
 * Revision 1.5  1996/04/12  22:09:01  ccobb
 * From Borland C++: now requires xh support.
 *
 * Revision 1.4  1996/03/04  06:46:52  ccobb
 * This version uses only the EHSTATE structure for both current state and
 * saved states.  ehFatalJmp() and ehExitJmp() have been made separate
 * functions.  OS initialization was changed to elimiate calls to xalloc()
 * (not reliable enough) and make a linked list instead.
 *
 * Revision 1.3  1995/08/16  19:49:12  ccobb
 * Compiled with C++.
 *
 * Revision 1.2  1995/04/09  08:34:32  ccobb
 * Well, it works now!  I can't remember all the changes.
 *

***/
