/***

ehvolume.c -- Error Handler, set channel/volume values

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char *rcsid =
       "$Id: ehvolume.c,v 1.3 1995/08/16 19:49:03 ccobb Exp $";

#include "ehvolume.h"

EHBEGINEXTERNC
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
EHENDEXTERNC

#include "ehchnvol.h"

#if !defined(tolower)
int  tolower A((int c));
#endif

#if !defined(toupper)
int  toupper A((int c));
#endif

/***
ehVolume() is called early during
initialization and therefore is
not an Error Handler compliant
function.
***/

int
ehVolume A1(const char *, pcEhVolume)
{
   int iRetVal = 0;
   int iChan =  0; /* by default, operate on channel 0 */
   int  iVol = -1;
   int  iFun = -1;
   int  iMem = -1;
   int  iOut = -1;
   char *pc;
   char *pcMem;
   char *pcVolume;
   char *pcSetting;

   /* make temp working buffer for environment variable */
   if ((pc = pcMem = (char *)calloc(strlen(pcEhVolume) + 1, (size_t)1)) == NULL)
   {
      fprintf(EHSTDERR, "calloc() failed\n");
      fflush(EHSTDERR);
      iRetVal = -3;
      goto Exit;
   }
   pcVolume = strcpy(pc, pcEhVolume);

   /* chop out sections */
   while ((pcSetting = strtok(pcVolume, "- \"")) != NULL)
   {

      pcVolume = NULL;   /* for strtok() */

      // printf("ehVolume: calling EhSetVal(%s,...)\n", pcSetting);
      if (ehSetVal(pcSetting, &iChan, &iVol, &iFun, &iMem, &iOut))
      {
         fprintf(EHSTDERR, "bad EHVOLUME option: %s\n", pcSetting);
         fflush(EHSTDERR);
         continue;
      }
   } /* end while */

Exit:
   if (pcMem)
      free (pcMem);
   return iRetVal;
}


int
ehSetVal A6(char *,  pcSetting,
             int *,  piChan,
             int *,  piVol,
             int *,  piFun,
             int *,  piMem,
             int *,  piOut)
{
   char cChan;
   char cVol;
   int  iRetVal = 0;

   // puts("ehSetVal: entering");

   if (*pcSetting == '-')
   {
      ++pcSetting;
   }
   switch (*pcSetting)
   {
   case 'c' : /*** check for channel ***/
      cChan = *(pcSetting + 1);
      if (cChan == '@')
      {
         *piChan = -1; /* apply to all channels */
      }
      else if (isdigit((int)cChan))
      {
         *piChan = cChan - '0';
      }
      else if (isalpha((int)cChan))
      {
         if (isupper((int)cChan))
            cChan = (char)tolower((int)cChan);
         for (*piChan = SZARR(apcEhChan); (*piChan)--; )
         {
            if (*apcEhChan[*piChan] == cChan)
            {
              break;
            }
         }
      }
      if (cChan != '@' && (*piChan < 0 || *piChan >= SZARR(apcEhChan)))
      {
         fprintf(EHSTDERR, "invalid channel: %s\n", pcSetting);
         fflush(EHSTDERR);
         iRetVal = 1;
         goto Exit;
      }
      break;

   case 'v' : /*** check for volume ***/
      cVol = (char)toupper((int)*(pcSetting + 1));
      if (isdigit((int)cVol))
      {
         *piVol = cVol - '0';
      }
      else
      {
         for (*piVol = SZARR(apcEhVol); (*piVol)--; )
         {
            if (*apcEhVol[*piVol] == cVol)          
            {
               break;
            }
         }
      }
      if (*piVol < 0 || *piVol >= SZARR(apcEhVol))
      {
         fprintf(EHSTDERR, "invalid volume: %s\n", pcSetting);
         fflush(EHSTDERR);
         iRetVal = 2;
         goto Exit;
      }
      /* set the volume on the specified channel(s) */
      if (*piChan < 0)
      {
         for (*piChan = SZARR(apcEhChan); (*piChan)--; )
         {
            ehSetVol(*piChan, *piVol);
         }
      }
      else
      {
         ehSetVol(*piChan, *piVol);
      }
      break;

   case 'f' : /*** check for function tracing ***/
      *piFun = atoi(pcSetting + 1);
      /* set function tracing on the specified channel(s) */
      if (*piChan < 0)
      {
         for (*piChan = SZARR(apcEhChan); (*piChan)--; )
         {
            ehSetFun(*piChan, *piFun);
         }
      }
      else
      {
         ehSetFun(*piChan, *piFun);
      }
      break;

   case 'm' : /*** check for memory tracing ***/
      *piMem = atoi(pcSetting + 1);
      /* set memory tracing on the specified channel(s) */
      if (*piChan < 0)
      {
         for (*piChan = SZARR(apcEhChan); (*piChan)--; )
         {
            ehSetMem(*piChan, *piMem);
         }
      }
      else
      {
         ehSetMem(*piChan, *piMem);
      }
      break;

   case 'o' : /*** select output file ***/
      *piOut = atoi(pcSetting + 1);
      /* set output file setting */
      ehSetOut(*piOut);
      break;

   default:
      fprintf(EHSTDERR, "Unrecognized setting: %s\n", pcSetting);
      fflush(EHSTDERR);
      break;

   }
Exit:
   // puts("ehSetVal: leaving");

   return (iRetVal);
}



