/***

ehwldcrd.c -- Error Handler, filename wild card Support

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char*rcsid =
       "$Id: ehwldcrd.c,v 1.2 1995/12/04 21:46:38 ccobb Exp $";

/*** system headers ***/

#include "ehwldcrd.h"

EHBEGINEXTERNC

#include <string.h>

#if ehAMIGA

#if ehSASC
#include <pragmas/dos_pragmas.h>
#endif

#if ehMX
#include <utility/tagitem.h> /* struct TagItem */
#include <dos/exall.h>       /* struct ExAllData */
#include <pragmas/dos_lib.h>
#endif

#if ehDCC
#endif

#if ehGNUC
#include <utility/tagitem.h> /* struct TagItem */
#include <dos/exall.h>       /* struct ExAllData */
#endif

#include <dos/dos.h>
#include <clib/dos_protos.h>

extern struct Library *DOSBase;

#endif /* ehAMIGA */

EHENDEXTERNC

/*** portability headers ***/
#include "eherrs.h"
#include "ehdynarr.h"

EHFILENAME("ehwldcrd.c");

/*** project headers ***/

/*** local (subsystem) headers ***/

/* make these static so we can free them later */
static size_t szArgcSav;
static char **ppcArgvSav;
static size_t szAllocated;
static char  *pcAllocated;

static short hasWildCards   A((const char *));
static void  addMatches     A((char*pcArgv));
static void  freeArgvMemory A((void));

/***

This function receives pointers to argc and argv.
It scans argv to see if there are any wild cards.
If there are none, we're done.  If there are,
argv is re-allocated and re-populated to resolve
any wild cards and replace the argv entries with
unambigous file names.

***/

void
ehExpWildCards A2(int *, piArgc, 
                  char ***, pppcArgv)
{
   int iArgc;
   char **ppcArgv;
   char **ppcNewArg;
   char *pcMark;

   EHTRACE(ehExpWildCards);

#if ehNIX
   EHLEAVE(); /* no expansion is necessary */
#endif

   /*** are there any wildcards? ***/
   for (iArgc = *piArgc, ppcArgv = *pppcArgv; iArgc--; ++ppcArgv)
   {
      /*** does ppcArgv have any wild cards? ***/
      if (hasWildCards(*ppcArgv))
         break;
   }
   if (iArgc < 0) /* loop was exhausted with no wildcards found */
      EHLEAVE(); /* no expansion is necessary */

   /*** if there are, we need to re-allocate argv ***/

   for (iArgc = 0, ppcArgv = *pppcArgv; iArgc < *piArgc; ++iArgc, ++ppcArgv)
   {
      if (hasWildCards(*ppcArgv))
      {
         /* add zero or more entries to ppvArgv */
         addMatches(*ppcArgv);
      }
      else /* no wildcards */
      {
         /* just copy entry over */
         ppcNewArg = (char**)ehDynArr(
               (void**)&ppcArgvSav, &szArgcSav, sizeof(char*));
         if (ppcNewArg == NULL) EHTHROW_LEAVE(EH_DYNARR);
         *ppcNewArg = *ppcArgv;

         /* mark as not allocated */
         pcMark = (char*)ehDynArr(
               (void**)&pcAllocated, &szAllocated, sizeof(char));
         if (pcMark == NULL) EHTHROW_LEAVE(EH_DYNARR);
         *pcMark = '\0';
      }
   }

   /* null-terminate the list */
   ppcNewArg = (char**)ehDynArr((void**)&ppcArgvSav, &szArgcSav, sizeof(char*));
   if (ppcNewArg == NULL) EHTHROW_LEAVE(EH_DYNARR);
   *ppcNewArg = NULL;

   /* mark as not allocated */
   pcMark = (char*)ehDynArr((void**)&pcAllocated, &szAllocated, sizeof(char));
   if (pcMark == NULL) EHTHROW_LEAVE(EH_DYNARR);
   *pcMark = '\0';

   if (ppcArgvSav)
   {
      *piArgc = (int)szArgcSav;
      *pppcArgv = ppcArgvSav;
      ehAtExit(freeArgvMemory);
   }
   EHLEAVE();
}

static
short
hasWildCards A1(const char *, pcArgv)
#if defined(HAVE_CONFIG_H) | ehBEOS
{pcArgv=pcArgv;return 0;}
#endif
#if ehAMIGA
{
   char *volatile pcDest = NULL;
   volatile LONG lIsWild = -1;
   LONG lLen;

   EHTRY(hasWildCards) {

      /* set up destination buffer */
      lLen = 3 * strlen(pcArgv) + 2;
      pcDest = (char*)calloc((int)lLen, 1);
      if (pcDest == NULL) EHTHROW(EH_CALLOC);

      lIsWild = ParsePatternNoCase((STRPTR)pcArgv, (STRPTR)pcDest, lLen);
      if (lIsWild < 0) EHTHROW(EH_PARSEPATTERNOCASE);

   } EHCATCH;

   if (pcDest) free(pcDest);
   EHRETURN((short)(lIsWild == 1));
}
#endif /* ehAMIGA */


static
void
addMatches A1(char*,    pcArgv)
#if defined(HAVE_CONFIG_H) | ehBEOS
{pcArgv=pcArgv;}
#endif
#if ehAMIGA
{
   struct AnchorPath * volatile pAnchorPath = NULL;
   char **ppcNewArgv;
   LONG lError = 0;
   char *pcMark;

   EHTRY(addMatches) {

      pAnchorPath = NEW(struct AnchorPath);
      if (pAnchorPath == NULL) EHTHROW(EH_NEWMACRO);

      lError = MatchFirst((STRPTR)pcArgv, pAnchorPath);
      while (lError == 0)
      {
         EHWRN1("found: [%s]", pAnchorPath->ap_Info.fib_FileName);

         /* add slot in argv for file name */
         ppcNewArgv = (char**)ehDynArr((void**)&ppcArgvSav,
               &szArgcSav, sizeof(char*));
         if (ppcNewArgv == NULL) EHTHROW(EH_DYNARR);

         /* duplicate the string */
         *ppcNewArgv = (char*)calloc(
               strlen(pAnchorPath->ap_Info.fib_FileName)+1,1);
         if (*ppcNewArgv == NULL) EHTHROW(EH_CALLOC);
         strcpy(*ppcNewArgv, pAnchorPath->ap_Info.fib_FileName);

         /* mark as allocated */
         pcMark = (char*)ehDynArr(
               (void**)&pcAllocated, &szAllocated, sizeof(char));
         if (pcMark == NULL) EHTHROW_LEAVE(EH_DYNARR);
         *pcMark = '\1';

         lError = MatchNext(pAnchorPath);
      }
      if (lError != ERROR_NO_MORE_ENTRIES)
      {
         ERR1("lError = %d", lError);
         EHTHROW(EH_MATCH);
      }

   } EHCATCH;

   if (pAnchorPath)
   {
      MatchEnd(pAnchorPath);
      free(pAnchorPath);
   }
   EHLEAVE();
}
#endif /* ehAMIGA */


static
void
freeArgvMemory A0()
{
   char  *pc;
   char **ppc;

   EHTRY(freeMemory) {

      EHASSERT_LEAVE(szArgcSav == szAllocated);

   } EHCATCH;

   pc  = pcAllocated + szAllocated;
   ppc = ppcArgvSav  + szArgcSav;

   while (szAllocated--)
   {
      EHWRN1("szAllocated = [%d]", szAllocated);
      --ppc;
      if (*--pc == '\1')
         free(*ppc);
   }
   if (pcAllocated) free(pcAllocated);
   if (ppcArgvSav)  free(ppcArgvSav);

   EHLEAVE();
}


