/***

t_trkmem.c -- MemTrack, test memory tracking functions

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

#include <string.h>

#include <eh.h>
#include "ehmemtrk.h"
#include "ehresini.h"

#define MTNOMEMTRK 1
#include <mt.h>

static void malloc_and_free();
static void malloc_and_no_free();
static void calloc_and_free();
static void calloc_and_no_free();
static void malloc_realloc_and_free();
static void malloc_realloc_and_no_free();
static void calloc_realloc_and_free();
static void calloc_realloc_and_no_free();
static void realloc_null_and_free();
static void realloc_null_and_no_free();
static void new_and_delete();
static void new_and_no_delete();

int
main(int argc, char**argv)
{
   EHMAIN(argc, argv) {

      malloc_and_free();
      malloc_and_no_free();

      calloc_and_free();
      calloc_and_no_free();

      malloc_realloc_and_free();
      malloc_realloc_and_no_free();

      calloc_realloc_and_free();
      calloc_realloc_and_no_free();

      realloc_null_and_free();
      realloc_null_and_no_free();

      new_and_delete();
      new_and_no_delete();

   } EHENDMAIN;

   EHDONE(ehError());
}

static
void
malloc_and_free()
{
   char *volatile pc = NULL;

   FUN_CATCH("malloc_and_free");

   pc = (char*)malloc(37);
   memset(pc, 'a', 37);

ExitSequence:
   if (pc) free(pc);
}


static
void
malloc_and_no_free()
{
   char *pc = NULL;
   EHTRACE(malloc_and_no_free);
   pc = (char*)malloc(37);
   memset(pc, 'b', 37);
}


static
void
calloc_and_free()
{
   char *volatile pc = NULL;

   FUN_CATCH("calloc_and_free");

   pc = (char*)calloc(3,7);
   memset(pc, 'c', 3*7);

ExitSequence:
   if (pc) free(pc);
}


static
void
calloc_and_no_free()
{
   char *pc = NULL;
   EHTRACE(calloc_and_no_free);
   pc = (char*)calloc(3,7);
   memset(pc, 'd', 3*7);
}


static
void
malloc_realloc_and_free()
{
   char *volatile pc = NULL;

   FUN_CATCH("malloc_realloc_and_free");

   pc = (char*)malloc(37);
   memset(pc, 'e', 37);
   pc = (char*)realloc(pc,3377);
   memset(pc, 'f', 3377);

ExitSequence:
   if (pc) free(pc);
}


static
void
malloc_realloc_and_no_free()
{
   char *pc = NULL;
   EHTRACE(malloc_realloc_and_no_free);
   pc = (char*)malloc(37);
   memset(pc, 'g', 37);
   pc = (char*)realloc(pc,3377);
   memset(pc, 'h', 3377);
}


static
void
calloc_realloc_and_free()
{
   char *volatile pc = NULL;

   FUN_CATCH("calloc_realloc_and_free");

   pc = (char*)calloc(33,77);
   memset(pc, 'i', 33*77);
   pc = (char*)realloc(pc,3377);
   memset(pc, 'j', 3377);

ExitSequence:
   if (pc) free(pc);
}


static
void
calloc_realloc_and_no_free()
{
   char *pc = NULL;
   EHTRACE(calloc_realloc_and_no_free);
   pc = (char*)calloc(33,77);
   memset(pc, 'k', 33*77);
   pc = (char*)realloc(pc,3377);
   memset(pc, 'l', 3377);
}


static
void
realloc_null_and_free()
{
   char *volatile pc = NULL;

   FUN_CATCH("realloc_null_and_free");

   pc = (char*)realloc(pc,3377);
   memset(pc, 'm', 3377);

ExitSequence:
   if (pc) free(pc);
}


static
void
realloc_null_and_no_free()
{
   char *pc = NULL;
   EHTRACE(realloc_null_and_no_free);
   pc = (char*)realloc(pc,3377);
   memset(pc, 'n', 3377);
}


static
void
new_and_delete()
{
   char *volatile pc = NULL;
   FUN_CATCH("new_and_delete");
   pc = new char[37];
   memset(pc, 'o', 37);
ExitSequence:
   if (pc) delete pc;
}


static
void
new_and_no_delete()
{
   char *pc = NULL;
   EHTRACE(new_and_no_delete);
   pc = new char[37];
   memset(pc, 'p', 37);
}


