/***

t_res.c -- Error Handler, test non-function-tracing resource tracking

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char*rcsid =
       "$Id: t_res.c,v 1.5 1996/03/04 06:46:59 ccobb Exp $";

#define _POSIX_SOURCE 1

#include "eh.h"
#include "eherrs.h"

EHBEGINEXTERNC
#include <stdio.h>
#include <string.h>
EHENDEXTERNC

#include "ehmemtrk.h"

static char*test_new    (bool proceed, int iCount);             /* n = 1 */
static char*test_malloc (bool proceed, int iCount);             /* n = 2 */
static char*test_calloc (bool proceed, int iCount);             /* n = 3 */
static char*test_realloc(bool proceed, char*pcMem, int iCount); /* n = 4 */
static char*test_realloc(bool proceed, char*pcMem, int iCount);   /* n = 5 */
static char*test_realloc(bool proceed, char*pcMem, int iCount);   /* n = 6 */
static void test_free   (bool proceed, char*pcMem);
static void test_delete (bool proceed, char*pcNewMem);

int 
main A2(
   int,    argc, 
   char**, argv)
{
   /* used in ExitSequence */
   volatile int  iDontFree = -1;
   volatile bool freeTwice = false;
   char *volatile pcNew = 0;
   char *volatile pcMalloc = 0;
   char *volatile pcCalloc = 0;
   char *volatile pcRealloc0 = 0;
   char *volatile pcRealloc1 = 0;
   char *volatile pcRealloc2 = 0;

   /* not used in ExitSequence */
   int   iForceFail = -1; // force failure on nth instance

   PGM_TRACE();

   ehGtArgs(&argc, &argv,
         EH_GA_MESSAGE, "-- test harness for memory & file resources --", 
         EH_GA_INT,   'f', &iForceFail, "force 'failure' on nth allocation",
         EH_GA_INT,   'd', &iDontFree,  "don't delete the memory",
         EH_GA_BOOL,  't', &freeTwice,  "free twice",
         0);

   pcNew      = test_new    (--iForceFail != 0, 100);           /* n = 1 */
   pcMalloc   = test_malloc (--iForceFail != 0, 200);           /* n = 2 */
   pcCalloc   = test_calloc (--iForceFail != 0, 300);           /* n = 3 */
   pcRealloc0 = test_realloc(--iForceFail != 0, NULL, 400);     /* n = 4 */
   pcRealloc1 = test_realloc(--iForceFail != 0, pcMalloc, 500); /* n = 5 */
   pcRealloc2 = test_realloc(--iForceFail != 0, pcCalloc, 600); /* n = 6 */

ExitSequence:
   if (pcRealloc0) test_free   (--iDontFree != 0, pcRealloc0);  /* n = 1 */
   if (pcRealloc1) test_free   (--iDontFree != 0, pcRealloc1);  /* n = 2 */
   if (pcRealloc2) test_free   (--iDontFree != 0, pcRealloc2);  /* n = 3 */
   if (pcNew)      test_delete (--iDontFree != 0, pcNew);       /* n = 6 */
   if (freeTwice) /* for test purposes */
   {
      if (pcCalloc)   test_free   (true, pcCalloc);
      if (pcMalloc)   test_free   (true, pcMalloc);
   }
   return ehError();
}

#include "ehmem.h"  /* track dynamic memory use without function tracing */
#include "ehfil.h"  /* track file stream use without function tracing */

static
char *           /* address of memory block or 0 if error or !proceed */
test_new(        /* allocate a block of memory */
   bool proceed, /* whether to really do it or not */
   int iCount)   /* how many bytes to allocate */
{
   if (!proceed)
      return 0;
   return(new char[iCount]);
}


static
char *           /* address of memory block or 0 if error or !proceed */
test_malloc(        /* allocate a block of memory */
   bool proceed, /* whether to really do it or not */
   int iCount)   /* how many bytes to allocate */
{
   if (!proceed)
      return 0;
   return((char*)malloc(iCount));
}


static
char *           /* address of memory block or 0 if error or !proceed */
test_calloc(        /* allocate a block of memory */
   bool proceed, /* whether to really do it or not */
   int iCount)   /* how many bytes to allocate */
{
   if (!proceed)
      return 0;
   return((char*)calloc(iCount,1));
}


static
char *           /* address of memory block or 0 if error or !proceed */
test_realloc(    /* allocate a block of memory */
   bool proceed, /* whether to really do it or not */
   char*pcMem,
   int iCount)   /* how many bytes to allocate */
{
   if (!proceed)
      return 0;
   return((char*)realloc(pcMem,iCount));
}


static
void
test_free(    /* allocate a block of memory */
   bool proceed, /* whether to really do it or not */
   char*pcMem)
{
   if (!proceed)
      return;
   free(pcMem);
   return;
}


static
void
test_delete(    /* allocate a block of memory */
   bool proceed, /* whether to really do it or not */
   char*pcMem)
{
   if (!proceed)
      return;
   delete [] pcMem;
   return;
}


